// $Id: action.cpp,v 1.134 2004/09/21 15:13:09 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <qapplication.h>
#include <qclipboard.h>
#include <qdir.h>
#include <qpointarray.h>

#include <iostream>
using namespace std;

#include "action.h"
#include "cil-idl.h"
#include "game.h"
#include "groupdialogs.h"
#include "highlight.h"
#include "main.h"
#include "player.h"
#include "polyline.h"
#include "surfacedialogs.h"
#include "tokendialogs.h"
#include "tokendrag.h"

extern Main *MW;
extern GameObj *Game;
extern QString TokenMIMEtype;

#include "tools/clone.xpm"
#include "tools/compound.xpm"
#include "tools/create.xpm"
#include "tools/eye.xpm"
#include "tools/flipleft.xpm"
#include "tools/flipright.xpm"
#include "tools/message.xpm"
#include "tools/move.xpm"
#include "tools/paste.xpm"
#include "tools/plot.xpm"
#include "tools/prop.xpm"
#include "tools/roll.xpm"
#include "tools/rotateccw.xpm"
#include "tools/rotatecw.xpm"
#include "tools/shuffle.xpm"
#include "tools/stack.xpm"
#include "tools/trash.xpm"
#include "tools/ungroup.xpm"

const QPixmap *TokenFlipLeftAction::Icon = 0;
const QPixmap *TokenFlipRightAction::Icon = 0;
const QPixmap *TokenRotateCWAction::Icon = 0;
const QPixmap *TokenRotateCCWAction::Icon = 0;
const QPixmap *TokenRenameAction::Icon = 0;
const QPixmap *TokenMoveAction::Icon = 0;
const QPixmap *TokenCloneAction::Icon = 0;
const QPixmap *TokenCreateAction::Icon = 0;
const QPixmap *TokenTrashAction::Icon = 0;
//const QPixmap *TokenPlotAction::Icon = 0;
//const QPixmap *TokenEndPlotAction::Icon = 0;
//const QPixmap *TokenCompoundAction::Icon = 0;
const QPixmap *TokenStackAction::Icon = 0;
const QPixmap *TokenShuffleAction::Icon = 0;
//const QPixmap *RollAction::Icon = 0;
//const QPixmap *MessageAction::Icon = 0;
const QPixmap *GroupCreateAction::Icon = 0;
const QPixmap *GroupTrashAction::Icon = 0;
const QPixmap *TokenRegroupAction::Icon = 0;
const QPixmap *GroupShowAction::Icon = 0;
const QPixmap *GroupHideAction::Icon = 0;
const QPixmap *GroupRegroupAction::Icon = 0;
const QPixmap *GroupRenameAction::Icon = 0;
const QPixmap *GroupSetPermissionAction::Icon = 0;
const QPixmap *SurfaceCreateAction::Icon = 0;
const QPixmap *SurfaceTrashAction::Icon = 0;
//const QPixmap *SurfaceCloneAction::Icon = 0;
const QPixmap *SurfaceRenameAction::Icon = 0;
const QPixmap *PlayerCreateAction::Icon = 0;
const QPixmap *PlayerTrashAction::Icon = 0;


//
// Action
//
Action::Action() : oid(Game->user->id()), step(0), update(true) {
}


Action::Action(unsigned int o) : oid(o), step(lastStep()), update(true) {
}


Action::~Action() {
}


bool Action::commit() {
   step = lastStep();
   unhighlight();
   return commitImp();
}


bool Action::undo() {
   step = 0;
   unhighlight();
   return undoImp();
}


void Action::begin() {
}


void Action::end() {
}


void Action::abort() {
}


void Action::preUndo() {
}


bool Action::canPrecede(const Action &) const { return true; }


void Action::setUpdate(bool u) { update = u; }


QValueList<QCanvas *> Action::canvasList() const {
   return QValueList<QCanvas *>();
}


QCanvasItemList Action::createdList() const { return QCanvasItemList(); }


Player *Action::owner() const { return (Player *)(Game->gid2g[oid]); }


QString Action::text() const { return "no textual representation"; }


QPoint Action::center() const { return QPoint(); }


bool Action::commitImp() { return true; }


bool Action::undoImp() { return true; }


void Action::highlight() {
}


void Action::unhighlight() {
}


void Action::focus() {
}


void Action::unfocus() {
}


unsigned int Action::stepForward() {
   switch(++step) {
      case 1:  focus();       break;
      case 2:  highlight();   break;
      case 3:  commitImp();   break;
      case 4:  unhighlight(); break;
      case 5:  unfocus();     break;
      default: step = 5;      break;
   }

   return step;
}


unsigned int Action::stepBackward() {
   switch(--step) {
      case 4:  focus();       break;
      case 3:  highlight();   break;
      case 2:  undoImp();     break;
      case 1:  unhighlight(); break;
      case 0:  unfocus();     break;
      default: step = 0;      break;
   }

   return step;
}


unsigned int Action::currentStep() const { return step; }


unsigned int Action::commitStep() const { return 3; }


unsigned int Action::undoStep() const { return 3; }


unsigned int Action:: firstStep() const { return 0; }


unsigned int Action::lastStep() const { return 5; }


QDomElement Action::prepareXML(QDomElement e) const {
   e.setAttribute("oid", oid);
   return e;
}


//
// ComboAction
//
ComboAction::ComboAction() {
}


ComboAction::ComboAction(unsigned int o) : Action(o) {
}


ComboAction::~ComboAction() {
   for (QValueList<Action *>::const_iterator i = list.begin();
    i != list.end(); ++i) delete *i;
}


unsigned int ComboAction::stepForward() {
   switch(++step) {
      case 1:
         focus();
         for (QValueList<Action *>::const_iterator i = list.begin();


          i != list.end(); ++i) (*i)->step++;
         break;
      case 2:
      case 3:
      case 4:
      case 5:
         for (QValueList<Action *>::const_iterator i = list.begin();
          i != list.end(); ++i) (*i)->stepForward();
         break;
      default:
         step = 5;
         break;
   }

   update();
   return step;
}


unsigned int ComboAction::stepBackward() {
   switch(--step) {
      case 4:  
         focus();
         for (QValueList<Action *>::const_iterator i = list.begin();
          i != list.end(); ++i) (*i)->step--;
         break;
      case 3:
      case 2:
      case 1:
      case 0:
         for (QValueList<Action *>::const_iterator i = list.begin();
          i != list.end(); ++i) (*i)->stepBackward();
         break;
      default:
         step = 0;
         break;
   }

   update();
   return step;
}



bool ComboAction::commit() {
   for (QValueList<Action *>::const_iterator i = list.begin();
    i != list.end(); ++i) (*i)->commit();
   update();
   step = lastStep();
   return true;
}


bool ComboAction::undo() {
   for (QValueList<Action *>::const_iterator i = list.fromLast();
    i != list.end(); --i) (*i)->undo();
   update();
   step = 0;
   return true;
}


void ComboAction::end() {
   for (QValueList<Action *>::const_iterator i = list.begin();
    i != list.end(); ++i) (*i)->end();
   step = lastStep();
   // FIXME: update(); ?
}


void ComboAction::abort() {
   for (QValueList<Action *>::const_iterator i = list.begin();
    i != list.end(); ++i) (*i)->abort();
   // FIXME: update(); ?
}


void ComboAction::preUndo() {
   for (QValueList<Action *>::const_iterator i = list.begin();
    i != list.end(); ++i) (*i)->preUndo();
}


bool ComboAction::isPermissible() const {
   for (QValueList<Action *>::const_iterator i = list.begin();
    i != list.end(); ++i) if (!((*i)->isPermissible())) return false;
   return true;
}


void ComboAction::append(Action *a) {
   list.append(a);
   a->setUpdate(false);
}


void ComboAction::prepend(Action *a) {
   list.prepend(a);
   a->setUpdate(false);
}


unsigned int ComboAction::count() const { return list.count(); }


QPixmap ComboAction::icon() const { return list.first()->icon(); }


QString ComboAction::name() const { return list.first()->name(); }


QValueList<QCanvas *> ComboAction::canvasList() const {
   // construct a list of QCanvases used by children of this
   // NOTE: list may contain duplicates
   QValueList<QCanvas *> canvases;
   for (QValueList<Action *>::const_iterator i = list.begin();
    i != list.end(); ++i) canvases += (*i)->canvasList();
   return canvases;
}


QCanvasItemList ComboAction::createdList() const {
   QCanvasItemList cl;
   for (QValueList<Action *>::const_iterator i = list.begin();
    i != list.end(); ++i) cl += (*i)->createdList();
   return cl;
}


Action *ComboAction::collapse() {
   Action *a = 0;
   switch(count()) {
      case 1:
         // collapse single-action combos
         a = list.first();
         list.remove(a);
         a->setUpdate(true);
     case 0:
         // goodbye, cruel world
         delete this;
         break;
     default:
         // return this otherwise
         a = this;
         break;
   }
   return a;
}


QPoint ComboAction::center() const {
   QPointArray points(list.count());

   unsigned int j = 0;
   for (QValueList<Action *>::const_iterator i = list.begin(); i != list.end();
    ++i, ++j) points.setPoint(j, (*i)->center());

   return points.boundingRect().center();
}


void ComboAction::focus() {
   for (QValueList<Action *>::const_iterator i = list.begin();
    i != list.end(); ++i) (*i)->focus();

   // FIXME: what to focus on? Combos could span Surfaces.
   /*
   QPointArray points(list.count());

   unsigned int j = 0;
   for (QValueList<Action *>::const_iterator i = list.begin();
    i != list.end(); ++i, ++j) {
      points.setPoint(j, (*i)->center());

   }

   points.boundingRect().center()
   */

   update();
}


QString ComboAction::text() const {
   QString s;
   for (QValueList<Action *>::const_iterator i = list.begin();
    i != list.end(); ++i) s += '\n' + (*i)->text();
   return s;
}


QDomElement ComboAction::dumpXML(QDomDocument &doc) const {
   return prepareXML(doc.createElement("combo"));
}


QDomElement ComboAction::prepareXML(QDomElement e) const {
   e = Action::prepareXML(e);

   QDomDocument doc = e.ownerDocument();

   // dump children in reverse
   for (QValueList<Action *>::const_iterator i = list.fromLast();
    i != list.end(); --i) e.appendChild((*i)->dumpXML(doc));
   return e;
}


void ComboAction::update() {
   // get a list of all canvases, and all touched canvases
   QValueList<QCanvas *> canvases = Game->c2s.keys();
   QValueList<QCanvas *> touched = canvasList();

   // update each canvas as necessary, iterating over the canvases
   // list, which contains no duplicates
   for (QValueList<QCanvas *>::const_iterator i = canvases.begin();
    i != canvases.end(); ++i)
     if (touched.find(*i) != touched.end()) (*i)->update();
}


//
// TokenAction
//
TokenAction::TokenAction(Token *t) : tid(t ? t->id() : 0), hlight(0) {
}


TokenAction::TokenAction(unsigned int o, unsigned int t) : Action(o), tid(t),
 hlight(0) {
}


TokenAction::~TokenAction() { unhighlight(); }


QValueList<QCanvas *> TokenAction::canvasList() const {
   QValueList<QCanvas *> list;
   list.append(((Token *)(Game->gid2g[tid]))->canvas());
   return list;
}


QPoint TokenAction::center() const {
   Token *t = (Token *)(Game->gid2g[tid]);
   return t->boundingRect().center();
}


void TokenAction::highlight() {
   Token *t = (Token *)(Game->gid2g[tid]);
   Player *p = (Player *)(Game->gid2g[oid]);

   QPen pen(Game->user->drawingColor(p), Game->user->drawingWidth(p));
   int margin = Game->user->drawingMargin(p);

   hlight = new Highlight(t, margin, pen);
   hlight->setZ(t->z());
   hlight->show();

   if (update) t->canvas()->update();
}


void TokenAction::unhighlight() {
   if (hlight && hlight->canvas()) {
      QCanvas *c = hlight->canvas();

      delete hlight;
      hlight = 0;
   
      if (update) c->update();
   }
}


void TokenAction::focus() {
   Token *t = (Token *)(Game->gid2g[tid]);
   Surface *s = Game->c2s[t->canvas()];

   QPoint c = center();
   s->center(c.x(), c.y());
   if (update) s->canvas()->update();
}


QDomElement TokenAction::prepareXML(QDomElement e) const {
   e = Action::prepareXML(e);
   e.setAttribute("tid", tid);
   return e;
}


//
// TokenFlipAction
//
TokenFlipAction::TokenFlipAction(Token *t, bool d) : TokenAction(t),
 direction(d) {
}


TokenFlipAction::TokenFlipAction(unsigned int o, unsigned int t, bool d) :
 TokenAction(o, t), direction(d) {
}


bool TokenFlipAction::isPermissible() const {
   return Game->gid2g[tid]->permission(Game->user, Perms::Flip);
}


bool TokenFlipAction::next() {
   Token *t = (Token *)(Game->gid2g[tid]);
   t->flipNext();
   Game->inplaydialog->update(0, t);
   if (update) t->canvas()->update();
   return true;
}


bool TokenFlipAction::previous() {
   Token *t = (Token *)(Game->gid2g[tid]);

   t->flipPrev();
   Game->inplaydialog->update(0, t);
   if (update) t->canvas()->update();
   return true;
}


QDomElement TokenFlipAction::dumpXML(QDomDocument &doc) const {
   return prepareXML(doc.createElement("tflip"));
}


QDomElement TokenFlipAction::prepareXML(QDomElement e) const {
   e = TokenAction::prepareXML(e);
   e.setAttribute("dir", direction);
   return e;
}


//
// TokenFlipLeftAction
//
TokenFlipLeftAction::TokenFlipLeftAction(Token *t) :
 TokenFlipAction(t, false) {
   if (!Icon) Icon = new const QPixmap((const char**)flipleft_xpm);
}


TokenFlipLeftAction::TokenFlipLeftAction(unsigned int o, unsigned int t) :
 TokenFlipAction(o, t, false) {
   if (!Icon) Icon = new const QPixmap((const char**)flipleft_xpm);
}


bool TokenFlipLeftAction::commitImp() { return previous(); }


bool TokenFlipLeftAction::undoImp() { return next(); }


QPixmap TokenFlipLeftAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


QString TokenFlipLeftAction::name() const { return "Flip Left"; }


//
// TokenFlipRightAction
//
TokenFlipRightAction::TokenFlipRightAction(Token *t) :
 TokenFlipAction(t, true) {
   if (!Icon) Icon = new const QPixmap((const char**)flipright_xpm);
}


TokenFlipRightAction::TokenFlipRightAction(unsigned int o, unsigned int t) :
 TokenFlipAction(o, t, true) {
   if (!Icon) Icon = new const QPixmap((const char**)flipright_xpm);
}


bool TokenFlipRightAction::commitImp() { return next(); }


bool TokenFlipRightAction::undoImp() { return previous(); }


QPixmap TokenFlipRightAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


QString TokenFlipRightAction::name() const { return "Flip Right"; }


//
// TokenRotateAction
//
TokenRotateAction::TokenRotateAction(Token *t, int dt) : TokenAction(t),
 dtheta(dt) {
}


TokenRotateAction::TokenRotateAction(unsigned int o, unsigned int t, int dt) :
 TokenAction(o, t), dtheta(dt) {
}


bool TokenRotateAction::commitImp() {
   Token *t = (Token *)(Game->gid2g[tid]);

   t->setTheta(t->theta() + dtheta);
   dtheta = -dtheta;
   if (update) t->canvas()->update();
   return true;
}


bool TokenRotateAction::undoImp() { return commitImp(); }


bool TokenRotateAction::isPermissible() const {
   return Game->gid2g[tid]->permission(Game->user, Perms::Rotate);
}


QDomElement TokenRotateAction::dumpXML(QDomDocument &doc) const {
   return prepareXML(doc.createElement("trotate"));
}


QDomElement TokenRotateAction::prepareXML(QDomElement e) const {
   e = TokenAction::prepareXML(e);
   e.setAttribute("dt", dtheta);
   return e;
}


//
// TokenRotateCWAction
//
TokenRotateCWAction::TokenRotateCWAction(Token *t) :
 TokenRotateAction(t, MW->dTheta()) {
   if (!Icon) Icon = new const QPixmap((const char **)rotatecw_xpm);
}


TokenRotateCWAction::TokenRotateCWAction(unsigned int o, unsigned int t,
 int dt) : TokenRotateAction(o, t, dt) {
   if (!Icon) Icon = new const QPixmap((const char **)rotatecw_xpm);
}


QPixmap TokenRotateCWAction::icon() const {
   if (!Icon) return QPixmap();
   return *Icon;
}


QString TokenRotateCWAction::name() const { return "Rotate CW"; }


//
// TokenRotateCCWAction
//
TokenRotateCCWAction::TokenRotateCCWAction(Token *t) :
 TokenRotateAction(t, -(MW->dTheta())) {
   if (!Icon) Icon = new const QPixmap((const char **)rotateccw_xpm);
}


TokenRotateCCWAction::TokenRotateCCWAction(unsigned int o, unsigned int t,
 int dt) : TokenRotateAction(o, t, dt) {
   if (!Icon) Icon = new const QPixmap((const char **)rotateccw_xpm);
}


QPixmap TokenRotateCCWAction::icon() const {
   if (!Icon) return QPixmap();
   return *Icon;
}


QString TokenRotateCCWAction::name() const { return "Rotate CCW"; }


//
// TokenMoveAction
//
TokenMoveAction::TokenMoveAction(Token *t, double xp, double yp, double zp)
 : TokenAction(t), x(xp), y(yp), z(zp), sid(Game->c2s[t->canvas()]->id()),
 here(0), there(0), line(0) {
   if (!Icon) Icon = new const QPixmap((const char**)move_xpm);
}


TokenMoveAction::TokenMoveAction(Token *t, QCanvas *dc, double xp, double yp,
 double zp) : TokenAction(t), x(xp), y(yp), z(zp), sid(Game->c2s[dc]->id()),
 here(0), there(0), line(0) {
   if (!Icon) Icon = new const QPixmap((const char**)move_xpm);
}


TokenMoveAction::TokenMoveAction(Token *t, QCanvas *dc) : TokenAction(t),
 x(t->x()), y(t->y()), z(t->z()), sid(Game->c2s[dc]->id()), here(0), there(0),
 line(0) {
   if (!Icon) Icon = new const QPixmap((const char**)move_xpm);
}


TokenMoveAction::TokenMoveAction(Token *t) : TokenAction(t), here(0), there(0),
 line(0) {
   if (!Icon) Icon = new const QPixmap((const char**)move_xpm);
}


TokenMoveAction::TokenMoveAction(unsigned int o, unsigned int t,
 unsigned int s, double xp, double yp, double zp) : TokenAction(o, t),
 x(xp), y(yp), z(zp), sid(s), here(0), there(0), line(0) {
   if (!Icon) Icon = new const QPixmap((const char**)move_xpm);
}


TokenMoveAction::~TokenMoveAction() { unhighlight(); }


bool TokenMoveAction::commitImp() {
   Token *t = (Token *)(Game->gid2g[tid]);
   QCanvas *c = Game->sid2s[sid]->canvas();

   if (c != t->canvas()) {
      QCanvas *oc = t->canvas();
      t->setCanvas(c);
      c = oc;
      sid = Game->c2s[oc]->id();
   }

   double oldx, oldy, oldz;
   oldx = t->x();
   t->setX(x);
   x = oldx;

   oldy = t->y();
   t->setY(y);
   y = oldy;

   oldz = t->z();
   t->setZ(z);
   z = oldz;

   if (update) {
      if (c != t->canvas()) c->update();
      t->canvas()->update();
   }

   Game->inplaydialog->update(2, t);
   Game->inplaydialog->update(3, t);

   return true;
}


bool TokenMoveAction::undoImp() { return commitImp(); }


bool TokenMoveAction::isPermissible() const {
   return Game->gid2g[tid]->permission(Game->user, Perms::Move);
}


QValueList<QCanvas *> TokenMoveAction::canvasList() const {
   QValueList<QCanvas *> list;
   list.append(((Token *)(Game->gid2g[tid]))->canvas());
   list.append(Game->sid2s[sid]->canvas());
   return list;
}


QPixmap TokenMoveAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


QString TokenMoveAction::name() const { return "Move"; }


void TokenMoveAction::begin() {
   Token *t = (Token *)(Game->gid2g[tid]);

   sid = Game->c2s[t->canvas()]->id();
   x = t->x();
   y = t->y();
   z = t->z();
}


void TokenMoveAction::end() {
   Token *t = (Token *)(Game->gid2g[tid]);

   Game->inplaydialog->update(2, t);
   Game->inplaydialog->update(3, t);

   step = lastStep();
}


QString TokenMoveAction::text() const {
   Token *t = (Token *)(Game->gid2g[tid]);

   return QString("%1: (%2,%3) --&gt; (%4,%5)").
    arg(t->name()).
    arg(x).
    arg(y).
    arg(t->x()).
    arg(t->y());
}


void TokenMoveAction::highlight() {
   Token *t = (Token *)(Game->gid2g[tid]);
   Surface *s = Game->c2s[t->canvas()];
   Player *p = (Player *)(Game->gid2g[oid]);

   QPen pen(Game->user->drawingColor(p), Game->user->drawingWidth(p));
   int margin = Game->user->drawingMargin(p);

   here = new Highlight(t, margin, pen);
   here->setZ(t->z());

   // the corner of the Token is offset by (-1,-1) from the corner of its
   // bounding rectangle
   QRect r(QPoint((int)x-1,(int)y-1), t->boundingRect().size());

   there = new Highlight(r, s->canvas(), margin, pen);
   there->setZ(t->z());

   line = new QCanvasLine(s->canvas());
   line->setPen(pen);
   line->setPoints(
    here->boundingRect().center().x(),
    here->boundingRect().center().y(),
    there->boundingRect().center().x(),
    there->boundingRect().center().y());
   line->setZ(t->z()-1);

   here->show();
   there->show();
   line->show();   

   if (update) s->canvas()->update();
}


void TokenMoveAction::unhighlight() {
   if (here && here->canvas()) {
      QCanvas *c = here->canvas();

      delete here;
      delete there;
      delete line;
      here = 0;
      there = 0;
      line = 0;
   
      if (update) c->update();
   }
}


QPoint TokenMoveAction::center() const {
   Token *t = (Token *)(Game->gid2g[tid]);

   // the src rectangle
   QRect src(t->boundingRect());

   // the dest rectangle
   QRect dest(src);
   dest.moveTopLeft(QPoint((int)x,(int)y) - QPoint(1,1));

   // focus on dest if src and dest are not on the same canvas
   if (t->canvas() != Game->sid2s[sid]->canvas()) return dest.center(); 

   Surface *s = Game->c2s[t->canvas()];

   // the visible rectangle
   QRect vis(s->contentsX(), s->contentsY(),
    s->contentsWidth(), s->contentsHeight());

   // check whether src and dest are in the visible rectangle, and if so
   // maintain the current center
   if (vis.contains(src) && vis.contains(dest)) return vis.center();
   
   // check whether src and dest fit in the visible area,
   // and if so focus on their bounding box 
   QRect bound(src | dest);
   if (bound.width() <= vis.width() && bound.height() <= vis.height()) 
    return bound.center();

   // otherwise focus on dest 
   return dest.center();
}


QDomElement TokenMoveAction::dumpXML(QDomDocument &doc) const {
   return prepareXML(doc.createElement("tmove"));
}


QDomElement TokenMoveAction::prepareXML(QDomElement e) const {
   e = TokenAction::prepareXML(e);
   e.setAttribute("sid", sid);
   e.setAttribute("ox", x);
   e.setAttribute("oy", y);
   e.setAttribute("oz", z);
   return e;
}


//
// TokenCreateAction
//
TokenCreateAction::TokenCreateAction(TokenType *d, Group *g) :
 TokenAction(0), gid(g ? g->id() : 0), sid(0), type(d->type) {
   tid = Game->maxID++;
   if (!Icon) Icon = new const QPixmap((const char **)create_xpm);
}


TokenCreateAction::TokenCreateAction(Token *t, Group *g) : TokenAction(0),
 gid(g ? g->id() : 0), sid(0), type(t->type()), Name(t->name()) {
   tid = Game->maxID++;
   if (!Icon) Icon = new const QPixmap((const char **)create_xpm);
}


TokenCreateAction::TokenCreateAction(unsigned int o, unsigned int t) :
 TokenAction(o, t), sid(0) {
   if (!Icon) Icon = new const QPixmap((const char **)create_xpm);
}


Token *TokenCreateAction::created() const {
   return (Token *)(Game->gid2g[tid]);
}


bool TokenCreateAction::commitImp() {
   Group *g = (Group *)(Game->gid2g[gid]);
   QCanvas *c = Game->sid2s[sid]->canvas();
   TokenType *d = Game->tTypes[type];

   Token *t = new Token(Game, g, c, (int)x, (int)y, (int)z, 0,
    d->name, d->type, d->side.first().id(), tid);
   Game->gid2g[tid] = t;

   t->show();

   Game->inplaydialog->insert(t);
   Game->groupsdialog->insert(t);

   if (update) c->update();
   return true;
}


bool TokenCreateAction::undoImp() {
   Token *t = (Token *)(Game->gid2g[tid]);
   QCanvas *c = t->canvas();

   Game->inplaydialog->remove(t);
   Game->groupsdialog->remove(t);
   delete t;
   Game->gid2g.remove(tid);

   if (update) c->update();
   return true;
}


void TokenCreateAction::preUndo() {
   if (!sid) {
      Token *t = (Token *)(Game->gid2g[tid]);
   
      sid = Game->c2s[t->canvas()]->id();
      gid = t->parent()->id();
      type = t->type();
      x = t->x();
      y = t->y();
      z = t->z();
   }
}


bool TokenCreateAction::isPermissible() const {
   return Game->gid2g[gid]->permission(Game->user, Perms::Insert);
}


QValueList<QCanvas *> TokenCreateAction::canvasList() const {
   QValueList<QCanvas *> list;
   list.append(Game->sid2s[sid]->canvas());
   return list;
}


QCanvasItemList TokenCreateAction::createdList() const {
   QCanvasItemList list;
   list.append(created());
   return list;
}


QPixmap TokenCreateAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


QString TokenCreateAction::name() const { return "New Token"; }


void TokenCreateAction::begin() {
   Group *g = (Group *)(Game->gid2g[gid]);
   TokenType *d = Game->tTypes[type];

   // create the new Token on scratch; call end() to drop it at its destination
   Token *t = new Token(Game, g, Game->scratch, 0, 0, 0, 0,
    d->name, d->type, d->side.first().id(), tid);
   Game->gid2g[tid] = t;

   t->show();
}


void TokenCreateAction::end() {
   Token *t = (Token *)(Game->gid2g[tid]);

   Game->inplaydialog->update(2, t);
   Game->inplaydialog->update(3, t);

   step = lastStep();
}


void TokenCreateAction::abort() {
   // delete the token only in the case that it is still on the scratch canvas
   Token *t = (Token *)(Game->gid2g[tid]);

   if (t->canvas() == Game->scratch) {
      Game->gid2g.remove(tid);
      delete t;
   }
}


QPoint TokenCreateAction::center() const {
   if (Game->gid2g.contains(tid)) return TokenAction::center();
   else return QPoint((int)x,(int)y) +
    Game->tTypes[type]->side.first().rect().center();
}


void TokenCreateAction::highlight() {
   if (Game->gid2g.contains(tid)) TokenAction::highlight();
   else {
      Player *p = (Player *)(Game->gid2g[oid]);
      Surface *s = Game->sid2s[sid];
      
      int margin = Game->user->drawingMargin(p);
      QPen pen(Game->user->drawingColor(p), Game->user->drawingWidth(p));
     
      // the bounding rectangle is 2 pixels wider and higher than the
      // Token's pixmap, and the Token corner is offset by (-1,-1) from
      // the corner of the bounding rectangle
      QRect r(QPoint((int)x-1,(int)y-1),
       Game->tTypes[type]->side.first().rect().size() + QSize(2,2));
 
      hlight = new Highlight(r, s->canvas(), margin, pen);
      hlight->setZ(z);
      hlight->show();

      if (update) hlight->canvas()->update();
   }
}


void TokenCreateAction::focus() {
   if (Game->gid2g.contains(tid)) TokenAction::focus();
   else {
      Surface *s = Game->sid2s[sid];
      
      QPoint c = center();
      s->center(c.x(), c.y());
      if (update) s->canvas()->update();
   }
}


QDomElement TokenCreateAction::dumpXML(QDomDocument &doc) const {
   return TokenAction::prepareXML(doc.createElement("tcreate"));
}


//
// TokenCloneAction
//
TokenCloneAction::TokenCloneAction(Token *c, Group *g) :
 TokenCreateAction(c, g), copyof(c->id()), up(c->up()), theta(c->theta()),
 here(0), there(0), line(0) {
   // reserve an id for the clone
   tid = Game->maxID++;

   if (!Icon) Icon = new const QPixmap((const char **)clone_xpm);
}


TokenCloneAction::TokenCloneAction(Token *c, Group *g, QCanvas *dc, double xp,
 double yp, double zp) : TokenCreateAction(c, g), copyof(c->id()), up(c->up()),
 theta(c->theta()), here(0), there(0), line(0) {
   // reserve an id for the clone
   tid = Game->maxID++;

   // set location
   sid = Game->c2s[dc]->id();
   x = xp;
   y = yp;
   z = zp;

   if (!Icon) Icon = new const QPixmap((const char**)clone_xpm);
}


TokenCloneAction::TokenCloneAction(unsigned int o, unsigned int t,
 unsigned int c) : TokenCreateAction(o, t), copyof(c), here(0), there(0),
 line(0) {
   if (!Icon) Icon = new const QPixmap((const char**)clone_xpm);
}


TokenCloneAction::~TokenCloneAction() { unhighlight(); }


void TokenCloneAction::abort() { Action::abort(); }


bool TokenCloneAction::commitImp() {
   Group *g = (Group *)(Game->gid2g[gid]);
   QCanvas *cv = Game->sid2s[sid]->canvas();
   Token *c = (Token *)(Game->gid2g[copyof]);

   Token *t = new Token(*c, tid);
   Game->gid2g[tid] = t;

   if (cv != t->canvas()) t->setCanvas(cv);
   t->reparent(g);
   Game->inplaydialog->insert(t);
   Game->groupsdialog->insert(t);

   t->setX(x);
   t->setY(y);
   t->setZ(z);

   t->show();

   if (update) t->canvas()->update();
   return true;
}


bool TokenCloneAction::undoImp() {
   Token *t = (Token *)(Game->gid2g[tid]);
   Token *c = (Token *)(Game->gid2g[copyof]);
   QCanvas *cv = t->canvas();
   
   Game->inplaydialog->remove(t);
   Game->groupsdialog->remove(t);
   delete t;
   Game->gid2g.remove(tid);

   if (update) {
      if (cv != c->canvas()) cv->update();
      c->canvas()->update();
   }
   return true;
}


void TokenCloneAction::preUndo() {
   if (!sid) {
      Token *t = (Token *)(Game->gid2g[tid]);
      
      sid = Game->c2s[t->canvas()]->id();
      gid = t->parent()->id();
      x = t->x();
      y = t->y();
      z = t->z();
   }
}


bool TokenCloneAction::isPermissible() const {
   return Game->gid2g[copyof]->permission(Game->user, Perms::Clone);
}


QValueList<QCanvas *> TokenCloneAction::canvasList() const {
   QValueList<QCanvas *> list;
   list.append(((Token *)(Game->gid2g[copyof]))->canvas());
   list.append(Game->sid2s[sid]->canvas());
   return list;
}


QPixmap TokenCloneAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


QString TokenCloneAction::name() const { return "Clone"; }


void TokenCloneAction::begin() {
   Token *c = (Token *)(Game->gid2g[copyof]);
   Group *g = (Group *)(Game->gid2g[gid]);

   Token *t = new Token(*c, tid);
   Game->gid2g[tid] = t;

   sid = Game->c2s[t->canvas()]->id();

   t->reparent(g);
   Game->inplaydialog->insert(t);
   Game->groupsdialog->insert(t);

   t->show();
}


QPoint TokenCloneAction::center() const {
   // FIXME: should the center involve both the clone and the original?
   if (Game->gid2g.contains(tid)) return TokenAction::center();
   else {
      Token *c = (Token *)(Game->gid2g[copyof]);
      QRect b = c->boundingRect();
      return QPoint((int)x,(int)y) + b.center() - b.topLeft();
   }
}


void TokenCloneAction::highlight() {
   Token *cp = (Token *)(Game->gid2g[copyof]);
   QCanvas *c = cp->canvas();
   
   QRect r(cp->boundingRect());

   if (Game->gid2g.contains(tid)) {
      Token *t = (Token *)(Game->gid2g[tid]);
      r.moveTopLeft(t->boundingRect().topLeft());
   }
   // the corner of the Token is offset by (-1,-1) from the corner of its
   // bounding rectangle
   else r.moveTopLeft(QPoint((int)x-1,(int)y-1));
     
   Player *p = (Player *)(Game->gid2g[oid]);

   QPen pen(Game->user->drawingColor(p), Game->user->drawingWidth(p));
   int margin = Game->user->drawingMargin(p);

   here = new Highlight(cp, margin, pen);
   here->setZ(cp->z());

   there = new Highlight(r, c, margin, pen);
   there->setZ(cp->z());

   line = new QCanvasLine(c);
   pen.setStyle(Qt::DashLine);
   line->setPen(pen);
   line->setPoints(
    here->boundingRect().center().x(),
    here->boundingRect().center().y(),
    there->boundingRect().center().x(),
    there->boundingRect().center().y());
   line->setZ(cp->z()-1);

   here->show();
   there->show();
   line->show();   

   if (update) c->update();
}


void TokenCloneAction::unhighlight() {
   if (here && here->canvas()) {
      QCanvas *c = here->canvas();

      delete here;
      delete there;
      delete line;
      here = 0;
      there = 0;
      line = 0;
      
      if (update) c->update();
   }
}


void TokenCloneAction::focus() {
   if (Game->gid2g.contains(tid)) TokenAction::focus();
   else {
      Token *cp = (Token *)(Game->gid2g[copyof]);
      Surface *s = Game->sid2s[sid];
      
      QPoint c(cp->boundingRect().center());
      s->center(c.x(), c.y());
      if (update) s->canvas()->update();
   }
}


QDomElement TokenCloneAction::dumpXML(QDomDocument &doc) const {
   return prepareXML(doc.createElement("tclone"));
}


QDomElement TokenCloneAction::prepareXML(QDomElement e) const {
   e = TokenCreateAction::prepareXML(e);
   e.setAttribute("cid", copyof);
   return e;
}


//
// TokenTrashAction
//
TokenTrashAction::TokenTrashAction(Token *t) : TokenAction(t),
 sid(Game->c2s[t->canvas()]->id()), gid(t->parent()->id()), type(t->type()),
 up(t->up()), x(t->x()), y(t->y()), z(t->z()), theta(t->theta()),
 Name(t->name()) {
   if (!Icon) Icon = new const QPixmap((const char**)trash_xpm);
}


TokenTrashAction::TokenTrashAction(unsigned int o, unsigned int t,
 unsigned int s, unsigned int g, double ox, double oy, double oz, int a,
 const QString &n, unsigned int tp, unsigned int u) : TokenAction(o, t),
 sid(s), gid(g), type(tp), up(u), x(ox), y(oy), z(oz), theta(a), Name(n) {
   if (!Icon) Icon = new const QPixmap((const char**)trash_xpm);
}


bool TokenTrashAction::commitImp() {
   Token *t = (Token *)(Game->gid2g[tid]);
   QCanvas *c = t->canvas();

   Game->inplaydialog->remove(t);
   Game->groupsdialog->remove(t);

   delete t;
   Game->gid2g.remove(tid);

   if (update) c->update();
   return true;
}


bool TokenTrashAction::undoImp() {
   Token *t = new Token(Game,
    (Group *)(Game->gid2g[gid]),
    Game->sid2s[sid]->canvas(),
    (int)x, (int)y, (int)z, theta, Name, type, up, tid);

   Game->gid2g[tid] = t;

   Game->groupsdialog->insert(t);
   Game->inplaydialog->insert(t);

   t->show();

   if (update) t->canvas()->update();
   return true;
}


bool TokenTrashAction::isPermissible() const {
   return Game->gid2g.contains(tid) ?
    Game->gid2g[tid]->permission(Game->user, Perms::Delete) :
    Game->gid2g[gid]->permission(Game->user, Perms::Insert);
}


QValueList<QCanvas *> TokenTrashAction::canvasList() const {
   QValueList<QCanvas *> list;
   list.append(Game->sid2s[sid]->canvas());
   return list;
}


QPoint TokenTrashAction::center() const {
   if (Game->gid2g.contains(tid)) return TokenAction::center();
   else return QPoint((int)x,(int)y) +
    Game->tTypes[type]->side.first().rect().center();
}


void TokenTrashAction::highlight() {
   if (Game->gid2g.contains(tid)) TokenAction::highlight();
   else {
      Player *p = (Player *)(Game->gid2g[oid]);
      Surface *s = Game->sid2s[sid];

      int margin = Game->user->drawingMargin(p);
      QPen pen(Game->user->drawingColor(p), Game->user->drawingWidth(p));
     
      TokenPixmapList::const_iterator i;
      TokenPixmapList pl = Game->tTypes[type]->side;
      for (i = pl.begin(); i != pl.end() && (*i).id() != up; ++i);

      // the bounding rectangle is 2 pixels wider and higher than the
      // Token's pixmap, and the Token corner is offset by (-1,-1) from
      // the corner of the bounding rectangle
      QRect r(QPoint((int)x-1,(int)y-1), (*i).rect().size() + QSize(2,2));
 
      hlight = new Highlight(r, s->canvas(), margin, pen);
      hlight->setZ(z);
      hlight->show();

      if (update) hlight->canvas()->update();
   }
}


void TokenTrashAction::focus() {
   if (Game->gid2g.contains(tid)) TokenAction::focus();
   else {
      Surface *s = Game->sid2s[sid];

      QPoint c = center();
      s->center(c.x(), c.y());
      if (update) s->canvas()->update();
   }
}


QPixmap TokenTrashAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


QString TokenTrashAction::name() const { return "Delete Token"; }


QDomElement TokenTrashAction::dumpXML(QDomDocument &doc) const {
   return prepareXML(doc.createElement("ttrash"));
}


QDomElement TokenTrashAction::prepareXML(QDomElement e) const {
   e = TokenAction::prepareXML(e);
   e.setAttribute("sid", sid);
   e.setAttribute("gid", gid);
   e.setAttribute("type", type);
   e.setAttribute("name", Name);
   e.setAttribute("up", up);
   e.setAttribute("x", x);
   e.setAttribute("y", y);
   e.setAttribute("z", z);
   e.setAttribute("t", theta);
   return e;
}


//
// TokenRegroupAction
//
TokenRegroupAction::TokenRegroupAction(Token *t) : TokenAction(t),
 gid(t->parent()->id()) {
   if (!Icon) Icon = new const QPixmap((const char**)ungroup_xpm);
}


TokenRegroupAction::TokenRegroupAction(Token *t, Group *p) : TokenAction(t),
 gid(p->id()) {
   if (!Icon) Icon = new const QPixmap((const char**)ungroup_xpm);
}


TokenRegroupAction::TokenRegroupAction(unsigned int o, unsigned int t,
 unsigned int p) : TokenAction(o, t),  gid(p) {
   if (!Icon) Icon = new const QPixmap((const char**)ungroup_xpm);
}


bool TokenRegroupAction::commitImp() {
   Token *t = (Token *)(Game->gid2g[tid]);
   Group *p = (Group *)(Game->gid2g[gid]);   

   Group *tmp = t->parent();
   t->reparent(p);
   Game->inplaydialog->update(4, t);
   Game->groupsdialog->reparent(t);
   gid = tmp->id();
   return true;
}


bool TokenRegroupAction::undoImp() { return commitImp(); }


void TokenRegroupAction::setParent(Group *p) { gid = p->id(); }


bool TokenRegroupAction::isPermissible() const {
   return Game->gid2g[tid]->permission(Game->user, Perms::Remove) &
    (Game->gid2g.contains(gid) ?
     Game->gid2g[gid]->permission(Game->user, Perms::Insert) : 1);
}


QPixmap TokenRegroupAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


QString TokenRegroupAction::name() const { return "Regroup"; }


QDomElement TokenRegroupAction::dumpXML(QDomDocument &doc) const {
   return prepareXML(doc.createElement("tregroup"));
}


QDomElement TokenRegroupAction::prepareXML(QDomElement e) const {
   e = TokenAction::prepareXML(e);
   e.setAttribute("gid", gid);
   return e;
}


//
// TokenRenameAction
//
TokenRenameAction::TokenRenameAction(Token *t, const QString &n) :
 TokenAction(t), Name(n) {
   if (!Icon) Icon = new const QPixmap((const char**)prop_xpm);
}


TokenRenameAction::TokenRenameAction(unsigned int o, unsigned int t,
 const QString &n) : TokenAction(o, t), Name(n) {
   if (!Icon) Icon = new const QPixmap((const char**)prop_xpm);
}


bool TokenRenameAction::commitImp() {
   Token *t = (Token *)(Game->gid2g[tid]);

   QString tmp = t->name();
   t->setName(Name);
   Game->inplaydialog->update(1, t);
   Game->groupsdialog->rename(t);
   Name = tmp;
   return true;
}


bool TokenRenameAction::undoImp() { return commitImp(); }


bool TokenRenameAction::isPermissible() const {
   return Game->gid2g[tid]->permission(Game->user, Perms::Properties);
}


QPixmap TokenRenameAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


QString TokenRenameAction::name() const { return "Rename Token"; }


QDomElement TokenRenameAction::dumpXML(QDomDocument &doc) const {
   return prepareXML(doc.createElement("trename"));
}


QDomElement TokenRenameAction::prepareXML(QDomElement e) const {
   e = TokenAction::prepareXML(e);
   e.setAttribute("name", Name);
   return e;
}


//
// TokenStackAction
//
TokenStackAction::TokenStackAction() {
   if (!Icon) Icon = new const QPixmap((const char**)stack_xpm);
}


TokenStackAction::TokenStackAction(unsigned int o) : ComboAction(o) {
   if (!Icon) Icon = new const QPixmap((const char**)stack_xpm);
}


QString TokenStackAction::name() const { return "Stack"; }


QPixmap TokenStackAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


Action *TokenStackAction::collapse() { return this; }


QDomElement TokenStackAction::dumpXML(QDomDocument &doc) const {
   return ComboAction::prepareXML(doc.createElement("tstack"));
}


//
// TokenInvertAction
//
TokenInvertAction::TokenInvertAction() {
}


TokenInvertAction::TokenInvertAction(unsigned int o) : TokenStackAction(o) {
}


QString TokenInvertAction::name() const { return "Invert"; }


QDomElement TokenInvertAction::dumpXML(QDomDocument &doc) const {
   return ComboAction::prepareXML(doc.createElement("tinvert"));
}


//
// TokenShuffleAction
//
TokenShuffleAction::TokenShuffleAction() {
   if (!Icon) Icon = new const QPixmap((const char **)shuffle_xpm);
}


TokenShuffleAction::TokenShuffleAction(unsigned int o) : TokenStackAction(o) {
   if (!Icon) Icon = new const QPixmap((const char **)shuffle_xpm);
}


QString TokenShuffleAction::name() const { return "Shuffle"; }


QPixmap TokenShuffleAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


QDomElement TokenShuffleAction::dumpXML(QDomDocument &doc) const {
   return ComboAction::prepareXML(doc.createElement("tshuffle"));
}


//
// PolylineAction
//
PolylineAction::PolylineAction(Polyline *l) : lid(l ? l->id() : 0) {
}


PolylineAction::PolylineAction(unsigned int o, unsigned int l) : Action(o),
 lid(l) {
}


QDomElement PolylineAction::prepareXML(QDomElement e) const {
   e = Action::prepareXML(e);
   e.setAttribute("lid", lid);
   return e;
}


//
// PolylineBeginAction
//
PolylineBeginAction::PolylineBeginAction(Group *g, QCanvas *c, double xp,
 double yp, const QString &n, QPen p) : PolylineAction(0), gid(g->id()),
 sid(Game->c2s[c]->id()), x(xp), y(yp), Name(n), pen(p) {
   lid = Game->maxID++;
}


PolylineBeginAction::PolylineBeginAction(unsigned int o, unsigned int g,
 unsigned int s, unsigned int l, double xp, double yp, const QString &n,
 QPen p) : PolylineAction(o, l), gid(g), sid(s), x(xp), y(yp), Name(n), pen(p) {
}


Polyline *PolylineBeginAction::created() const {
   return (Polyline *)(Game->gid2g[lid]);
}


bool PolylineBeginAction::commitImp() {
   Group *g = (Group *)(Game->gid2g[gid]);
   QCanvas *c = Game->sid2s[sid]->canvas();
   
   Polyline *l = new Polyline(Game, Name, g, c, lid);
   Game->gid2g[lid] = l;

   l->setPen(pen);
   l->pushPoint(QPoint(x, y));
   l->show();

   if (update) c->update();
   return true;
}


bool PolylineBeginAction::undoImp() {
   Polyline *l = (Polyline *)(Game->gid2g[lid]);
   QCanvas *c = l->canvas();

   delete l;
   Game->gid2g.remove(lid);

   if (update) c->update();
   return true;
}


QPixmap PolylineBeginAction::icon() const {
   return QPixmap();
}


QString PolylineBeginAction::name() const { return "Begin Polyline"; }


bool PolylineBeginAction::isPermissible() const {
   return true;
} 


QDomElement PolylineBeginAction::dumpXML(QDomDocument &doc) const {
   return prepareXML(doc.createElement("plbegin"));
}


QDomElement PolylineBeginAction::prepareXML(QDomElement e) const {
   return e;
}


//
// PolylineExtendAction
//
PolylineExtendAction::PolylineExtendAction(Polyline *l, double xp, double yp,
 QPen p) : PolylineAction(l), x(xp), y(yp), pen(p) {
}


PolylineExtendAction::PolylineExtendAction(unsigned int o, unsigned int l,
 double xp, double yp, QPen p) : PolylineAction(o, l), x(xp), y(yp), pen(p) {
}


bool PolylineExtendAction::commitImp() {
   Polyline *l = (Polyline *)(Game->gid2g[lid]);
   QCanvas *c = l->canvas();

   l->setPen(pen);
   l->pushPoint(QPoint(x, y));
   l->show();   

   if (update) c->update();
   return true;
}


bool PolylineExtendAction::undoImp() {
   Polyline *l = (Polyline *)(Game->gid2g[lid]);
   QCanvas *c = l->canvas();

   l->popPoint();
   
   if (update) c->update();
   return true;
}


QPixmap PolylineExtendAction::icon() const {
   return QPixmap();
}


QString PolylineExtendAction::name() const { return "Extend Polyline"; }


bool PolylineExtendAction::isPermissible() const {
   return true;
} 


QDomElement PolylineExtendAction::dumpXML(QDomDocument &doc) const {
   return prepareXML(doc.createElement("plextend"));
}


QDomElement PolylineExtendAction::prepareXML(QDomElement e) const {
   return e;
}


//
// GroupAction
//
GroupAction::GroupAction(Group *g) : gid(g ? g->id() : 0) { 
}


GroupAction::GroupAction(unsigned int o, unsigned int g) : Action(o), gid(g) {
}


QDomElement GroupAction::prepareXML(QDomElement e) const {
   e = Action::prepareXML(e);
   e.setAttribute("gid", gid);
   return e;
}


//
// GroupCreateAction
//
GroupCreateAction::GroupCreateAction(const QString &n, Group *p) : 
 GroupAction(0), pid(p->id()), Name(n) {
   gid = Game->maxID++;
   if (!Icon) Icon = new const QPixmap((const char**)create_xpm);
}


GroupCreateAction::GroupCreateAction(unsigned int o, unsigned int g) :
 GroupAction(o, g) {
   if (!Icon) Icon = new const QPixmap((const char**)create_xpm);
}


bool GroupCreateAction::commitImp() {
   Group *p = (Group *)(Game->gid2g[pid]);
   Group *g = new Group(Game, Name, p, gid);
   Game->gid2g[gid] = g;

   g->reparent(p);
   Game->groupsdialog->insert(g);
   return true;
}


bool GroupCreateAction::undoImp() {
   Group *g = (Group *)(Game->gid2g[gid]);

   g->reparent(0);
   Game->groupsdialog->remove(g);
   delete g;
   Game->gid2g.remove(gid);

   return true;
}


Group *GroupCreateAction::created() const {
   return (Group *)(Game->gid2g[gid]);
}


bool GroupCreateAction::isPermissible() const {
   return Game->gid2g[pid]->permission(Game->user, Perms::Insert);
}


QPixmap GroupCreateAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}
        

QString GroupCreateAction::name() const { return "New Group"; }

        
QDomElement GroupCreateAction::dumpXML(QDomDocument &doc) const {
   return GroupAction::prepareXML(doc.createElement("gcreate"));
}


//
// GroupTrashAction
//
GroupTrashAction::GroupTrashAction(Group *g) : gid(g->id()),
 pid(g->parent() ? g->parent()->id() : 0) {
   // recursively trash children of group
   Group *gr;
   Player *pl;
   Token *tk;
   for (Group::const_iterator i = g->begin(); i != g->end(); ++i) {
      if ((tk = dynamic_cast<Token *>(*i)))
       append(new TokenTrashAction(tk));
      else if ((pl = dynamic_cast<Player *>(*i)))
       append(new PlayerTrashAction(pl));
      else if ((gr = dynamic_cast<Group *>(*i)))
       append(new GroupTrashAction(gr));
   }

   if (!Icon) Icon = new const QPixmap((const char**)trash_xpm);
}


GroupTrashAction::GroupTrashAction(unsigned int o, unsigned int g,
 unsigned int p, const QString &n) : ComboAction(o), gid(g), pid(p), Name(n) {
   if (!Icon) Icon = new const QPixmap((const char**)trash_xpm);
}


bool GroupTrashAction::commitImp() {
   ComboAction::commitImp();

   Group *g = (Group *)(Game->gid2g[gid]);

   g->reparent(0);   
   Game->groupsdialog->remove(g);
   delete g;
   Game->gid2g.remove(gid);

   return true;
}
        

bool GroupTrashAction::undoImp() {
   Group *p = (Group *)(Game->gid2g[pid]);
   Group *g = new Group(Game, Name, p, gid);
   Game->gid2g[pid] = g;
   Game->groupsdialog->insert(g);

   return ComboAction::undoImp();
}
        

bool GroupTrashAction::isPermissible() const {
   return (Game->gid2g.contains(gid) ? 
    Game->gid2g[gid]->permission(Game->user, Perms::Delete) : true) 
    & ComboAction::isPermissible(); 
}


QPixmap GroupTrashAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


QString GroupTrashAction::name() const { return "Delete Group"; }


QDomElement GroupTrashAction::dumpXML(QDomDocument &doc) const {
   return prepareXML(doc.createElement("gtrash"));
}


QDomElement GroupTrashAction::prepareXML(QDomElement e) const {
   e = ComboAction::prepareXML(e);
   e.setAttribute("gid", gid);
   e.setAttribute("pid", pid);
   e.setAttribute("name", Name);
   return e;
}


//
// GroupRegroupAction
//
GroupRegroupAction::GroupRegroupAction(Group *g) : GroupAction(g),
 pid(g->parent()->id()) {
   if (!Icon) Icon = new const QPixmap((const char **)ungroup_xpm);
}


GroupRegroupAction::GroupRegroupAction(unsigned int o, unsigned int g,
 unsigned int p) : GroupAction(o, g), pid(p) {
   if (!Icon) Icon = new const QPixmap((const char **)ungroup_xpm);
}


bool GroupRegroupAction::commitImp() {
   Group *g = (Group *)(Game->gid2g[gid]);
   Group *p = (Group *)(Game->gid2g[pid]);

   Group *tmp = g->parent();
   g->reparent(p);
   Game->groupsdialog->reparent(g);
   pid = tmp->id();
   return true;
}


bool GroupRegroupAction::undoImp() { return commitImp(); }


void GroupRegroupAction::begin() {
}


void GroupRegroupAction::setParent(Group *p) { pid = p->id(); }


bool GroupRegroupAction::isPermissible() const {
   return Game->gid2g[gid]->permission(Game->user, Perms::Remove) &
    (Game->gid2g.contains(pid) ?
     Game->gid2g[pid]->permission(Game->user, Perms::Insert) : true);
}


QPixmap GroupRegroupAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


QString GroupRegroupAction::name() const { return "Regroup"; }


QDomElement GroupRegroupAction::dumpXML(QDomDocument &doc) const {
   return prepareXML(doc.createElement("gregroup"));
}


QDomElement GroupRegroupAction::prepareXML(QDomElement e) const {
   e = GroupAction::prepareXML(e);
   e.setAttribute("pid", pid);
   return e;
}


//
// GroupRenameAction
//
GroupRenameAction::GroupRenameAction(Group *g, const QString &n) :
 GroupAction(g), Name(n) {
   if (!Icon) Icon = new const QPixmap((const char**)prop_xpm);
}


GroupRenameAction::GroupRenameAction(unsigned int o, unsigned int g,
 const QString &n) : GroupAction(o, g), Name(n) {
   if (!Icon) Icon = new const QPixmap((const char**)prop_xpm);
}


bool GroupRenameAction::commitImp() {
   Group *g = (Group *)(Game->gid2g[gid]);

   QString tmp = g->name();
   g->setName(Name);
   Game->groupsdialog->rename(g);
   Name = tmp;
   return true;
}


bool GroupRenameAction::undoImp() { return commitImp(); }


bool GroupRenameAction::isPermissible() const {
   return Game->gid2g[gid]->permission(Game->user, Perms::Properties);
}


QPixmap GroupRenameAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


QString GroupRenameAction::name() const { return "Rename Group"; }


QDomElement GroupRenameAction::dumpXML(QDomDocument &doc) const {
   return prepareXML(doc.createElement("grename"));
}


QDomElement GroupRenameAction::prepareXML(QDomElement e) const {
   e = GroupAction::prepareXML(e);
   e.setAttribute("name", Name);
   return e;
}


//
// GroupSetPermissionAction
//
GroupSetPermissionAction::GroupSetPermissionAction(Group *g, Player *p,
 unsigned int pm, int v) : GroupAction(g), pid(p->id()), perm(pm), val(v) {
   if (!Icon) Icon = new const QPixmap((const char**)prop_xpm);
}


GroupSetPermissionAction::GroupSetPermissionAction(unsigned int o,
 unsigned int g, unsigned int p, unsigned int pm, int v) :
 GroupAction(o, g), pid(p), perm(pm), val(v) {
   if (!Icon) Icon = new const QPixmap((const char**)prop_xpm);
}


bool GroupSetPermissionAction::commitImp() {
   Group *g = (Group *)(Game->gid2g[gid]);
   Player *p = (Player *)(Game->gid2g[pid]);

   int old = g->localPermission(p, perm);
   if (val != -1) g->setPermission(p, perm, val);
   else g->unsetPermission(p, perm);
   val = old;
   return true;
}


bool GroupSetPermissionAction::undoImp() { return commitImp(); }


bool GroupSetPermissionAction::isPermissible() const {
   return Game->gid2g[gid]->permission(Game->user, Perms::Properties);
}


QPixmap GroupSetPermissionAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


QString GroupSetPermissionAction::name() const {
   return "Set Group Permissions";
}


QDomElement GroupSetPermissionAction::dumpXML(QDomDocument &doc) const {
   // FIXME!
   return QDomElement(); 
}


QDomElement GroupSetPermissionAction::prepareXML(QDomElement e) const {
   // FIXME!
   return QDomElement();   
}


//
// GroupVisibilityAction
//
GroupVisibilityAction::GroupVisibilityAction(Group *g) : GroupAction(g) {
}


GroupVisibilityAction::GroupVisibilityAction(unsigned int o, unsigned int g) :
 GroupAction(o, g) {
}


bool GroupVisibilityAction::show() {
   Group *g = (Group *)(Game->gid2g[gid]);

   g->show();

   QValueList<QCanvas *> list = g->canvases();
   for (QValueList<QCanvas *>::const_iterator i = list.begin();
    i != list.end(); ++i) (*i)->update();

   showUpwards(g);
   return true;
}


bool GroupVisibilityAction::hide() {
   Group *g = (Group *)(Game->gid2g[gid]);

   g->hide();

   QValueList<QCanvas *> list = g->canvases();
   for (QValueList<QCanvas *>::const_iterator i = list.begin();
    i != list.end(); ++i) (*i)->update();

   hideUpwards(g);
   return true;
}


void GroupVisibilityAction::showUpwards(Group *g) const {
   for (Group::const_iterator i = g->begin(); i != g->end(); ++i)
    if (!(*i)->isVisible()) return;

   Game->groupsdialog->setVisibility(g, true);
   Group *parent = g->parent();
   if (parent) showUpwards(parent);
}


void GroupVisibilityAction::hideUpwards(Group *g) const {
   Game->groupsdialog->setVisibility(g, false);
   Group *parent = g->parent();
   if (parent) hideUpwards(parent);
}


//
// GroupShowAction
//
GroupShowAction::GroupShowAction(Group *g) : GroupVisibilityAction(g) {
   if (!Icon) Icon = new const QPixmap((const char**)eye_xpm);
}


GroupShowAction::GroupShowAction(unsigned int o, unsigned int g) :
 GroupVisibilityAction(o, g) {
   if (!Icon) Icon = new const QPixmap((const char**)eye_xpm);
}


bool GroupShowAction::commitImp() { return show(); }


bool GroupShowAction::undoImp() { return hide(); }


bool GroupShowAction::isPermissible() const {
   return Game->gid2g[gid]->permission(Game->user, Perms::Show);
}


QPixmap GroupShowAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


QString GroupShowAction::name() const { return "Show"; }


QDomElement GroupShowAction::dumpXML(QDomDocument &doc) const {
   return GroupAction::prepareXML(doc.createElement("gshow"));
}


//
// GroupHideAction
//
GroupHideAction::GroupHideAction(Group *g) : GroupVisibilityAction(g) {
   if (!Icon) Icon = new const QPixmap((const char**)eye_xpm);
}


GroupHideAction::GroupHideAction(unsigned int o, unsigned int g) :
 GroupVisibilityAction(o, g) {
   if (!Icon) Icon = new const QPixmap((const char**)eye_xpm);
}


bool GroupHideAction::commitImp() { return hide(); }


bool GroupHideAction::undoImp() { return show(); }


bool GroupHideAction::isPermissible() const {
   return Game->gid2g[gid]->permission(Game->user, Perms::Hide);
}


QPixmap GroupHideAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


QString GroupHideAction::name() const { return "Hide"; }


QDomElement GroupHideAction::dumpXML(QDomDocument &doc) const {
   return GroupAction::prepareXML(doc.createElement("ghide"));
}


//
// PlayerAction
//
PlayerAction::PlayerAction(Player *p) : pid(p ? p->id() : 0) {
}


PlayerAction::PlayerAction(unsigned int o, unsigned int p) : Action(o),
 pid(p) {
}


QDomElement PlayerAction::prepareXML(QDomElement e) const {
   e = Action::prepareXML(e);
   e.setAttribute("pid", pid);
   return e;
}


//
// PlayerCreateAction
//
PlayerCreateAction::PlayerCreateAction(const QString &n) : PlayerAction(0),
 Name(n) {
   pid = Game->maxID++;
   if (!Icon) Icon = new const QPixmap((const char**)create_xpm);
}


PlayerCreateAction::PlayerCreateAction(unsigned int o, unsigned int p) :
 PlayerAction(o, p) {
   if (!Icon) Icon = new const QPixmap((const char**)create_xpm);
}


bool PlayerCreateAction::commitImp() {
   Player *p = new Player(Game, Name, QString::null, pid);
   Game->gid2g[pid] = p;

   p->reparent(Game->root);
   Game->groupsdialog->insert(p);

   Game->groupsdialog->setVisibility(p, true);
   p->show();
   return true;
}


bool PlayerCreateAction::undoImp() {
   Player *p = (Player *)(Game->gid2g[pid]);

   p->reparent(0);
   Game->groupsdialog->remove(p);
   delete p;
   Game->gid2g.remove(pid);
   
   return true;
}


Player *PlayerCreateAction::created() const {
   return (Player *)(Game->gid2g[pid]);
}


bool PlayerCreateAction::isPermissible() const { return true; }


QPixmap PlayerCreateAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


QString PlayerCreateAction::name() const { return "New Player"; }


QDomElement PlayerCreateAction::dumpXML(QDomDocument &doc) const {
   return PlayerAction::prepareXML(doc.createElement("pcreate"));
}


//
// PlayerTrashAction
//
PlayerTrashAction::PlayerTrashAction(Player *p) : pid(p->id()),
 Name(p->name()), Pass(p->password()) {
   // recursively trash children of player
   Group *gr;
   Token *tk;
   for (Group::const_iterator i = p->begin(); i != p->end(); ++i) {
      if ((tk = dynamic_cast<Token *>(*i))) append(new TokenTrashAction(tk));
      else if ((gr = dynamic_cast<Group *>(*i)))
       append(new GroupTrashAction(gr));
   }

   if (!Icon) Icon = new const QPixmap((const char**)trash_xpm);
}


PlayerTrashAction::PlayerTrashAction(unsigned int o, unsigned int p,
 const QString &n, const QString &pw) : ComboAction(o), pid(p),
 Name(n), Pass(pw) {
   if (!Icon) Icon = new const QPixmap((const char**)trash_xpm);
}


bool PlayerTrashAction::commitImp() {
   ComboAction::commitImp();

   Player *p = (Player *)(Game->gid2g[pid]);

   p->reparent(0);
   Game->groupsdialog->remove(p);
   delete p;
   Game->gid2g.remove(pid);

   return true;
}


bool PlayerTrashAction::undoImp() {
   Player *p = new Player(Game, Name, Pass, pid);
   Game->gid2g[pid] = p;

   p->reparent(Game->root);
   Game->groupsdialog->insert(p);
   return ComboAction::undoImp();
}


bool PlayerTrashAction::isPermissible() const {
   return (Game->gid2g.contains(pid) ?
    Game->gid2g[pid]->permission(Game->user, Perms::Delete) : true) &
    ComboAction::isPermissible();
}


QPixmap PlayerTrashAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


QString PlayerTrashAction::name() const { return "Delete Player"; }


QDomElement PlayerTrashAction::dumpXML(QDomDocument &doc) const {
   return prepareXML(doc.createElement("ptrash"));
}


QDomElement PlayerTrashAction::prepareXML(QDomElement e) const {
   e = ComboAction::prepareXML(e);
   e.setAttribute("pid", pid);
   e.setAttribute("name", Name);
   e.setAttribute("password", Pass);   
   return e;
}


//
// PlayerRenameAction
//
PlayerRenameAction::PlayerRenameAction(Player *p, const QString &n) :
 GroupRenameAction(p, n) {
}


PlayerRenameAction::PlayerRenameAction(unsigned int o, unsigned int p,
 const QString &n) : GroupRenameAction(o, p, n) {
}


QString PlayerRenameAction::name() const { return "Rename Player"; }


QDomElement PlayerRenameAction::dumpXML(QDomDocument &doc) const {
   return prepareXML(doc.createElement("prename"));
}


QDomElement PlayerRenameAction::prepareXML(QDomElement e) const {
   e = Action::prepareXML(e);
   e.setAttribute("pid", gid);
   e.setAttribute("name", Name);
   return e;
}


//
// SurfaceAction
//
SurfaceAction::SurfaceAction(Surface *s) : sid(s ? s->id() : 0) {
}


SurfaceAction::SurfaceAction(unsigned int o, unsigned int s) : Action(o),
 sid(s) {
}


QDomElement SurfaceAction::prepareXML(QDomElement e) const {
   e = Action::prepareXML(e);
   e.setAttribute("sid", sid);
   return e;
}


//
// SurfaceCreateAction
//
SurfaceCreateAction::SurfaceCreateAction(const QString &n) : SurfaceAction(0),
 Name(n) {
   sid = Game->maxID++;
   if (!Icon) Icon = new const QPixmap((const char**)create_xpm);
}


SurfaceCreateAction::SurfaceCreateAction(unsigned int o, unsigned int s)
 : SurfaceAction(o, s) {
   if (!Icon) Icon = new const QPixmap((const char**)create_xpm);
}


bool SurfaceCreateAction::commitImp() {
   Surface *s = new Surface(Game, new QCanvas, Name, QString::null,
#ifdef Q_WS_WIN
    QImage(),
#else
    QPixmap(),
#endif
   sid);

   Game->sid2s[sid] = s;
   Game->c2s[s->canvas()] = s;

   s->setCaption(MW->title() + " - " + s->surfaceName());

   Game->surfacesdialog->insert(s);
   s->show();
   return true;
}


bool SurfaceCreateAction::undoImp() {
   Surface *s = Game->sid2s[sid];
   Game->c2s.remove(s->canvas());

   Game->surfacesdialog->remove(s);

   delete s;
   Game->sid2s.remove(sid);
   return true;
}


Surface *SurfaceCreateAction::created() const { return Game->sid2s[sid]; }


bool SurfaceCreateAction::isPermissible() const { return true; }


QPixmap SurfaceCreateAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


QString SurfaceCreateAction::name() const { return "New Surface"; }


QDomElement SurfaceCreateAction::dumpXML(QDomDocument &doc) const {
   return SurfaceAction::prepareXML(doc.createElement("screate"));
}


//
// SurfaceTrashAction
//
SurfaceTrashAction::SurfaceTrashAction(Surface *s) : sid(s->id()),
 Name(s->surfaceName()), background(s->backFile().fileName()) {
   // trash all children
   QValueList<Token *> list = s->tokens();
   for (QValueList<Token *>::const_iterator i = list.begin(); i != list.end();
    ++i) append(new TokenTrashAction(*i));

   if (!Icon) Icon = new const QPixmap((const char**)trash_xpm);
}


SurfaceTrashAction::SurfaceTrashAction(unsigned int o, unsigned int s,
 const QString &n, const QString &b) : ComboAction(o), sid(s), Name(n),
 background(b) {
   if (!Icon) Icon = new const QPixmap((const char**)trash_xpm);
}


bool SurfaceTrashAction::commitImp() {
   ComboAction::commitImp();
   
   Surface *s = Game->sid2s[sid]; 
   Game->c2s.remove(s->canvas());

   Game->surfacesdialog->remove(s);
   delete s;   
   Game->sid2s.remove(sid);
   
   return true;
}

   
bool SurfaceTrashAction::undoImp() {
   // FIXME: This is ugly. Loading the background image should happen
   // somewhere from which errors can easily be reported, or in the
   // Surface constructor.

   QFileInfo bf(background);
   bf.setFile(Game->boxfile.dir(), bf.fileName());

#ifdef Q_WS_WIN
   // Load background as QImage on Windows.
   QImage bg;
#else
   // Load background as QPixmap on all sane platforms.
   QPixmap bg;
#endif

   // FIXME: this could fail!
   bg.load(bf.filePath());

   Surface *s = new Surface(Game, new QCanvas, Name, background, bg, sid);
   Game->sid2s[sid] = s;
   Game->c2s[s->canvas()] = s;

   // FIXME: this should happen in the Surface constructor?
   s->setCaption(MW->title() + " - " + s->surfaceName());

   Game->surfacesdialog->insert(s);
   s->show();
   return ComboAction::undoImp();
}


bool SurfaceTrashAction::isPermissible() const {
   // FIXME: eventually make surfaces ownable by players?
   return ComboAction::isPermissible();
}


QPixmap SurfaceTrashAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


QString SurfaceTrashAction::name() const { return "Delete Surface"; }


QDomElement SurfaceTrashAction::dumpXML(QDomDocument &doc) const {
   return prepareXML(doc.createElement("strash"));
}


QDomElement SurfaceTrashAction::prepareXML(QDomElement e) const {
   e = ComboAction::prepareXML(e);
   e.setAttribute("sid", sid);
   e.setAttribute("name", Name);
   e.setAttribute("background", background);
   return e;
}


//
// SurfaceRenameAction
//
SurfaceRenameAction::SurfaceRenameAction(Surface *s, const QString &n) :
 SurfaceAction(s), Name(n) {
   if (!Icon) Icon = new const QPixmap((const char**)prop_xpm);
}


SurfaceRenameAction::SurfaceRenameAction(unsigned int o, unsigned int s,
 const QString &n) : SurfaceAction(o, s), Name(n) {
   if (!Icon) Icon = new const QPixmap((const char**)prop_xpm);
}


bool SurfaceRenameAction::commitImp() {
   Surface *s = Game->sid2s[sid];

   QString tmp = s->surfaceName();
   s->setSurfaceName(Name);
   s->setCaption(MW->title() + " - " + s->surfaceName());
   Game->surfacesdialog->rename(s);
   Name = tmp;
   return true;
}


bool SurfaceRenameAction::undoImp() { return commitImp(); }


bool SurfaceRenameAction::isPermissible() const { return true; }


QPixmap SurfaceRenameAction::icon() const {
   if (!Icon) return QPixmap();
   else return *Icon;
}


QString SurfaceRenameAction::name() const { return "Rename Surface"; }


QDomElement SurfaceRenameAction::dumpXML(QDomDocument &doc) const {
   return prepareXML(doc.createElement("srename"));
}


QDomElement SurfaceRenameAction::prepareXML(QDomElement e) const {
   e = SurfaceAction::prepareXML(e);
   e.setAttribute("name", Name);
   return e;
}
