// $Id: bijection.h,v 1.8 2004/07/16 17:45:47 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __BIJECTION_H_
#define __BIJECTION_H_

#include <qmap.h>

template <class A, class B> class Bijection {
   public: 
      void insert(const A &a, const B &b) {
         a2b.insert(a, b); b2a.insert(b, a);
      }
      void insert(const B &b, const A &a) { insert(a, b); }  
      void remove(const A &a) { b2a.remove(a2b[a]); a2b.remove(a); }
      void remove(const B &b) { a2b.remove(b2a[b]); b2a.remove(b); }
      A &operator[](const B &b) { return b2a[b]; }
      B &operator[](const A &a) { return a2b[a]; }
      const A &operator[](const B &b) const { return b2a[b]; }
      const B &operator[](const A &a) const { return a2b[a]; }
      bool contains(const A &a) const { return a2b.contains(a); }
      bool contains(const B &b) const { return b2a.contains(b); }
      unsigned int count() const { return a2b.count(); }
      bool isEmpty() const { return a2b.isEmpty(); }
      QValueList<A> left() const { return a2b.keys(); }
      QValueList<B> right() const { return b2a.keys(); }

   private:
      QMap<A, B> a2b;
      QMap<B, A> b2a;
};

#endif /* __BIJECTION_H_ */
