// $Id: crayon.cpp,v 1.2 2004/09/03 05:10:29 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2002, 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "crayon.h"


//
// CrayonSegment
//
CrayonSegment::CrayonSegment(QPoint p1, QPoint p2, QCanvas *c,
 CrayonScrawl *s) : QCanvasLine(c), scrawl(s) {
   setPoints(p1.x(), p1.y(), p2.x(), p2.y());
}


int CrayonSegment::rtti() const { return CrayonSegment::RTTI; }


//
// CrayonScrawl
//
CrayonScrawl::CrayonScrawl(QCanvas *c) : canvas(c), dot(0) {
}


CrayonScrawl::~CrayonScrawl() {
   /*
   for (QValueList<CrayonSegment *>::const_iterator i = segments.begin();
    i != segments.end(); ++i) delete *i;
   */
}


void CrayonScrawl::pushPoint(QPoint p2) {
   if (segments.count()) {
      QPoint p1(segments.top()->endPoint());
      CrayonSegment *seg = new CrayonSegment(p1, p2, canvas, this);
      seg->setPen(pen);
      segments.push(seg);
   }
   else if (dot) {
      QPoint p1(int(dot->x()), int(dot->y()));
      CrayonSegment *seg = new CrayonSegment(p1, p2, canvas, this);
      seg->setPen(pen);
      segments.push(seg);
      delete dot;
      dot = 0;      
   }
   else {
      dot = new QCanvasEllipse(pen.width(), pen.width(), canvas);
      dot->setPen(pen);
      dot->setX(p2.x());
      dot->setY(p2.y());
   }
}


QPoint CrayonScrawl::popPoint() {
   QPoint p2;

   if (segments.count()) {
      CrayonSegment *seg = segments.pop();
      p2 = seg->endPoint();
      
      if (segments.count() == 1) {
         QPoint p1 = seg->startPoint();
         dot = new QCanvasEllipse(pen.width(), pen.width(), canvas);
         dot->setPen(pen);
         dot->setX(p1.x());
         dot->setY(p1.y());
      }

      delete seg;
   }
   else if (dot) {
      p2 = QPoint(int(dot->x()), int(dot->y()));
      delete dot;
      dot = 0;
   }

   return p2;
}


void CrayonScrawl::setPen(QPen p) { pen = p; }


void CrayonScrawl::show() {
   if (segments.count())
    for (QValueStack<CrayonSegment *>::const_iterator i = segments.begin();
     i != segments.end(); ++i) (*i)->show();
   else if (dot) dot->show();
}


void CrayonScrawl::hide() {
   if (segments.count())
    for (QValueStack<CrayonSegment *>::const_iterator i = segments.begin();
     i != segments.end(); ++i) (*i)->hide();
   else if (dot) dot->hide(); 
}


void CrayonScrawl::setX(double x) {
   if (segments.count()) {
      double ox = segments.top()->endPoint().x() - x;
      for (QValueStack<CrayonSegment *>::const_iterator i = segments.begin();
       i != segments.end(); ++i) (*i)->setX((*i)->x()+ox);
   }
   else if (dot) dot->setX(x);
}


void CrayonScrawl::setY(double y) {
   if (segments.count()) {
      double oy = segments.top()->endPoint().y() - y;
      for (QValueStack<CrayonSegment *>::const_iterator i = segments.begin();
       i != segments.end(); ++i) (*i)->setY((*i)->y()+oy);
   }
   else if (dot) dot->setY(y);
}


void CrayonScrawl::setZ(double z) {
   if (segments.count()) 
    for (QValueStack<CrayonSegment *>::const_iterator i = segments.begin();
     i != segments.end(); ++i) (*i)->setZ((*i)->z()+z);
   else if (dot) dot->setZ(z);
}
