// $Id: game.cpp,v 1.30 2004/08/24 14:54:18 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//#include <qiconset.h>

#include "groupdialogs.h"
#include "history.h"
#include "messagedialog.h"
#include "selectionhandler.h"
#include "surfacedialogs.h"
#include "tokendialogs.h"
#include "game.h"

//
// GameObj
//
GameObj::GameObj() : scratch(new QCanvas), user(0), root(0),
 title(), boxid(), boxver(),
 boxfile(), savefile(), scenariofile(),
 changed(false),
 alldialog(new TokenListDialog),
 inplaydialog(new TokenListViewDialog(this, QIconSet(), "In Play")),
 selecteddialog(new TokenListViewDialog(this, QIconSet(), "Selected")),
 groupsdialog(new GroupTreeDialog(this)),
 surfacesdialog(new SurfaceListDialog(this)),
 messagedialog(new MessageDialog(this)),
 historydialog(new HistoryDialog(this)),
 selector(new SelectionHandler(this)),
 maxID(0) {
}


GameObj::~GameObj() {
   delete root;
   delete scratch;
   delete alldialog;
   delete inplaydialog;
   delete selecteddialog;
   delete groupsdialog;
   delete surfacesdialog;
   delete messagedialog;
   delete historydialog;

   QValueList<Surface *> sl = sid2s.values();
   for (QValueList<Surface *>::const_iterator i = sl.begin();
    i != sl.end(); ++i) delete *i;

   delete selector;

   for (QMapConstIterator<unsigned int, TokenType *> i = tTypes.begin();
    i != tTypes.end(); ++i) delete *i;
}
