// $Id: game.h,v 1.29 2004/08/24 14:54:18 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __GAME_H_
#define __GAME_H_

#include <qfileinfo.h>

#include "action.h"
#include "logstack.h"
#include "token.h"
#include "surface.h"

class QCanvas;

class Groupable;
class GroupTreeDialog;
class HistoryDialog;
class MessageDialog;
class Player;
class RootGroup;
class TokenListDialog;
class TokenListViewDialog;
class SurfaceListDialog;
class SelectionHandler;


class GameObj {
   public:
      GameObj();
      ~GameObj();
      
      QCanvas *scratch;
      LogStack done;
      ActionStack undone;
      Player *user;
      RootGroup *root;
      QMap<QCanvas *, Surface *> c2s;
      QMap<unsigned int, Surface *> sid2s;
      QMap<unsigned int, Groupable *> gid2g;
      QMap<unsigned int, TokenPixmap> tPics;
      QMap<unsigned int, TokenType *> tTypes;

      QString title, boxid, boxver;
      QFileInfo boxfile, savefile, scenariofile;      
      bool changed;

      TokenListDialog *alldialog;
      TokenListViewDialog *inplaydialog, *selecteddialog;
      GroupTreeDialog *groupsdialog;
      SurfaceListDialog *surfacesdialog;
      MessageDialog *messagedialog;
      HistoryDialog *historydialog;

      SelectionHandler *selector;

      unsigned int maxID;
};


#endif /* __GAME_H_ */
