// $Id: group.h,v 1.36 2004/08/12 22:16:38 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __GROUP_H_
#define __GROUP_H_

#include <qdom.h>
#include <qmap.h>

class Token;
class Group;
class Player;
class GameObj;
class QCanvas;


class Perms {
   public:
      enum { Show, Hide, Move, Clone, Rotate, Flip, Delete,
             Insert, Remove, Info, Properties, PNum };

      int get(const Player *, unsigned int) const;
      QMap<unsigned int, bool> get(const Player *) const;
      void set(const Player *, unsigned int, bool);
      void set(const Player *, const QString &);
      void unset(const Player *, unsigned int);
      QString toString(const Player *) const;
      bool contains(const Player *) const;

   private:
      QMap<const Player *, QMap<unsigned int, bool> > p2p;
};


class Groupable {
   public:
      Groupable(GameObj *, const QString &, Group *, unsigned int);
      virtual ~Groupable();
      QString name() const;
      void setName(const QString &);
      QString notes() const;
      void setNotes(const QString &);
      virtual bool permission(const Player *, unsigned int) const = 0;
      virtual Player *owner();
      Group *parent() const;
      void reparent(Group *);
      virtual QDomElement dumpXML(QDomDocument &) const = 0;
      unsigned int id() const;
      virtual void show() = 0;
      virtual void hide() = 0;
      virtual bool isVisible() const = 0;

   protected:
      GameObj *game;
      QString Name, Notes;
      Group *Parent;
      unsigned int ID;
};


class Group : public QValueList<Groupable *>, public Groupable {
   public:
      Group(GameObj *, const QString &, Group *g, unsigned int i);
      virtual ~Group();
      bool permission(const Player *, unsigned int) const;
      int localPermission(const Player *, unsigned int) const;
      bool inheritedPermission(const Player *, unsigned int) const;
      void setPermission(const Player *, unsigned int, bool);
      void unsetPermission(const Player *, unsigned int);
      void setPermissions(const Player *, const QString &);
      virtual QDomElement dumpXML(QDomDocument &) const;
      bool selected() const;
      void setSelected(bool s);
      virtual bool isVisible() const;
      void setVisibility(bool v);
      void setVisibility(Player *p, bool v);
      virtual void show();
      virtual void hide();
      QValueList<Token *> tokens() const;
      QValueList<QCanvas *> canvases() const;

   protected:
      bool Selected;
      Perms perms;
      QMap<const Player *, bool> Visible;
};


class RootGroup : public Group {
   public:
      RootGroup(GameObj *g, unsigned int  i);
      virtual QDomElement dumpXML(QDomDocument &doc) const;
      QValueList<Player *> players() const;
};

#endif /* __GROUP_H_ */
