// $Id: groupdialogs.h,v 1.39 2004/08/03 22:06:41 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2002, 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __GROUPDIALOGS_H_
#define __GROUPDIALOGS_H_

#include <qdialog.h>
#include <qdragobject.h>
#include <qlistview.h>
#include <qmap.h>

#include "bijection.h"
#include "editview.h"
#include "group.h"
#include "tabpage.h"

class ComboAction;
class GameObj;
class Player;


class GroupTreeView : public EditView {
   Q_OBJECT

   friend class GroupTreeItem;
   friend class GroupTreeCheckItem;

   public:
      GroupTreeView(GameObj *g, QWidget *parent = 0, const char *name = 0);
      virtual void setSelected(QListViewItem *i, bool select);

   signals:
      void rootSelected(bool);

   protected:
      virtual void contentsDragEnterEvent(QDragEnterEvent *e);
      virtual void contentsDragMoveEvent(QDragMoveEvent *e);
      virtual void contentsDragLeaveEvent(QDragLeaveEvent *e);
      virtual void contentsDropEvent(QDropEvent *e);
      virtual void contentsMouseMoveEvent(QMouseEvent *e);
      virtual void contentsMousePressEvent(QMouseEvent *e);
      virtual void contentsMouseReleaseEvent(QMouseEvent *e);

   protected slots:
      virtual void rename();

   private slots:
      void openGroup();

   private:
      bool pressed;
      QPoint presspos;
      QTimer *autoopen_timer;
      QListViewItem *oldCurrent, *dropItem;  
      static const unsigned int autoopen_time = 750;
      GameObj *game;
};


class GroupTreeCheckItem : public QCheckListItem {
   friend class GroupTreeView;

   public:
      GroupTreeCheckItem(GroupTreeView *parent, const QString &text);
      GroupTreeCheckItem(QListViewItem *parent, const QString &text);
      ~GroupTreeCheckItem();

   protected:
      virtual void activate();

   private:
      void constructVisibilityAction(Group *g, ComboAction *combo) const;
};


class GroupTreeItemDrag : public QStoredDrag {
   Q_OBJECT

   public:
      GroupTreeItemDrag(QListViewItem *i, QWidget *dragSource = 0,
       const char *name = 0);
      static bool canDecode(const QMimeSource *m);
      static bool decode(const QMimeSource *m, QListViewItem *&i);
};


class GroupTreeDialog : public TabPage {
   Q_OBJECT

   friend class GroupTreeView;
   friend class GroupTreeCheckItem;

   public:
      GroupTreeDialog(GameObj *g, QWidget *parent = 0, const char *name = 0,
       WFlags f = 0);
      Group *currentGroup() const;
      void setVisibility(Group *g, bool v);

   public slots:
      void insert(Groupable *g);
      void remove(Groupable *g);
      void reparent(Groupable *g);
      void rename(Groupable *g);

   private slots:
      void createGroup();
      void deleteGroup();
      void showPermissions();
      void showPermissions(QListViewItem *i);
      void showProperties();
      void changedSelection(QListViewItem *i);
      void showContextMenu(QListViewItem *i, const QPoint &p);
      

   private:
      void constructVisibilityAction(Group *g, ComboAction *combo,
       bool on) const;

      GroupTreeView *view;
      Bijection<Groupable *, QListViewItem *> g2i;
      QPushButton *nb, *tb;
      GameObj *game;
};


class GroupPermListItem : public QListViewItem {
   friend class QListView;

   public:
      GroupPermListItem(QListView *parent, const QString &name);
      virtual int width(const QFontMetrics &fm, const QListView *lv,
       int column) const;
      virtual void paintCell(QPainter *p, const QColorGroup &cg, int column,
       int width, int align);
      void setState(int column, unsigned int state);
      unsigned int state(int column) const;
      void setInherited(int column, bool in);
      bool inherited(int column) const;
      bool isEnabled() const;
      void setEnabled(bool e);

   protected:
      virtual void activate();

   private:
      unsigned int State[Perms::PNum];
      bool Inherited[Perms::PNum];
      bool enabled;
};


class GroupPropertiesDialog : public QDialog {
   Q_OBJECT

   public:
      static void editPerms(Group *g, QWidget *parent = 0);
      static void showPerms(Group *g, QWidget *parent = 0);

   private:
      GroupPropertiesDialog(Group *g, bool edit, QWidget *parent = 0,
       const char *name = 0, bool modal = true, WFlags f = 0);

      Group *group;
      QListView *view;
      QMap<QListViewItem *, Player *> i2p;

      static const QPixmap *Icon[Perms::PNum];
};


/*
class GroupCreateDialog : public QDialog {
   Q_OBJECT

   public:
      GroupCreateDialog(QWidget *parent = 0, const char *name = 0,
       bool modal = true, WFlags f = 0);
      static QString getName(QWidget *parent = 0);

   private:
      QLineEdit *ne;
      QComboBox *combo;
};
*/

#endif /* __GROUPDIALOGS_H_ */
