// $Id: history.h,v 1.6 2004/08/11 04:50:04 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __HISTORY_H_
#define __HISTORY_H_

#include <qlistbox.h>

#include "bijection.h"
#include "tabpage.h"

class Action;
class GameObj;

class QTimer;


class HistoryListItem : public QListBoxText {
   public:
      HistoryListItem(const QString &text, Action *a = 0);
      Action *action() const;

   private:
      Action *actionPtr;
};


class HistoryListBox : public QListBox {
   Q_OBJECT

   public:
      HistoryListBox(GameObj *g, QWidget *parent = 0, const char *name = 0,
       WFlags f = 0);
      void finalize();
      void logAction(Action *a);
      void unlogAction(Action *a);
      void abortAction(Action *a);

   public slots:
      void first();
      void last();
      void prev();
      void next();
      void rewind();
      void stop();
      void play();

   private slots:
      void stepBackwardOne();
      void stepForwardOne();
      void stepBackwardContinuous();
      void stepForwardContinuous();

   private:
      void pgUp();
      void pgDn();

      bool skipBackward(int target);
      bool skipForward(int target);

      void stopTimer();

      virtual void mousePressEvent(QMouseEvent *e);
      virtual void keyPressEvent(QKeyEvent *e);

      GameObj *game;
      QTimer *timer;
};


class HistoryDialog : public TabPage {
   Q_OBJECT

   public:
      HistoryDialog(GameObj *g, QWidget *parent = 0, const char *name = 0,
       WFlags f = 0);
      void finalize();
      void prev();
      void next();

   public slots:
      void logAction(Action *a);
      void unlogAction(Action *a);
      void abortAction(Action *a);      

   private:
      HistoryListBox *lb;
};

#endif /* __HISTORY_H_ */
