// $Id: logstack.cpp,v 1.4 2004/08/26 22:03:33 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "action.h"
#include "logstack.h"


//
// ActionStack
//
ActionStack::~ActionStack() { while (!isEmpty()) delete pop(); }


//
// LogStack
//
void LogStack::log(Action *a) {
   // undo until action can be pushed
   while (!isEmpty() && !top()->canPrecede(*a)) {
      Action *p = pop();
      p->undo();
      emit unlogged(p);
      delete p;
   }

   // do post for action at the top of the stack
   // FIXME: is post ever necessary? maybe remove it.
   //if (!isEmpty()) top()->post();

   // clear highlighting for previous action
   if (!isEmpty()) while (top()->stepForward() != top()->lastStep());

   // push action and notify whatever is listening
   push(a);
   emit changed(true);
   emit logged(a);
}


void LogStack::abort() {
   // delete the most recent action, probably because it failed
   if (!isEmpty()) {
      Action *a = pop();
      a->abort();
      emit changed(true);
      emit aborted(a);
      delete a;
   }
}


void LogStack::complete() {
   if (!isEmpty()) {
      Action *a = top();
      a->end();
      emit completed(a);
   }
}


QDomElement LogStack::dumpXML(QDomDocument &doc) {
   QDomElement e = doc.createElement("actions");

   // dump in reverse, treating LogStack as a list
   for (LogStack::const_iterator i = fromLast(); i != end(); --i)
    e.appendChild((*i)->dumpXML(doc));

   return e;
}
