// $Id: main.cpp,v 1.215 2004/09/01 20:47:01 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2002, 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#define INCLUDE_MENUITEM_DEF

#include <iostream>
using namespace std;

#include <qapplication.h>
#include <qcanvas.h>
#include <qdom.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qgrid.h>
#include <qimage.h>
#include <qinputdialog.h>
#include <qlayout.h>
#include <qmenubar.h>
#include <qmenudata.h>
#include <qmessagebox.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qtabwidget.h>
#include <qtextstream.h>
#include <qtoolbutton.h>
#include <qtooltip.h>
#include <qxml.h>

#include "action.h"
#include "token.h"
#include "tokendialogs.h"
#include "common.h"
#include "game.h"
#include "group.h"
#include "groupdialogs.h"
#include "history.h"
//#include "message.h"
#include "messagedialog.h"
#include "playerdialogs.h"
#include "preferences.h"
#include "preferencesdialog.h"
#include "properties.h"
//#include "roll.h"
#include "surface.h"
#include "surfacedialogs.h"
#include "tabdialog.h"
#include "tabpage.h"
#include "view.h"
#include "loadhandlers.h"

#include "main.h"

#include "tools/clone.xpm"
#include "tools/flipleft.xpm"
#include "tools/flipright.xpm"
#include "tools/group.xpm"
#include "tools/info.xpm"
#include "tools/move.xpm"
#include "tools/plot.xpm"
#include "tools/rotateccw.xpm"
#include "tools/rotatecw.xpm"
#include "tools/select.xpm"
#include "tools/trash.xpm"
#include "tools/wand.xpm"
#include "tools/zoomin.xpm"
#include "tools/zoomout.xpm"

#include "tools/pencil.xpm"

PreferenceMap Pref;
Main *MW = 0;
GameObj *Game = 0;

QString GroupTreeItemMIMEtype;
QString TokenMIMEtype;
QString TabMIMEtype;


//
// Main
// 
Main::Main(QWidget *parent, const char *name, WFlags f) :
 QWidget(parent, name, f), toolgroup(this) {
   // set global main window pointer
   MW = this;

   // read preferences
   loadDefaultPrefs();
   if (!loadUserPrefs())
    inform("Cannot find user preferences. Using defaults instead.");

   // build the menubar and popup menu
   mb = new QMenuBar(this);
   
   // file
   QPopupMenu *file = new QPopupMenu;
   mb->insertItem("&File", file, M_FILE);
   file->insertItem("&New...", this, SLOT(fileNew()), CTRL+Key_N,
    M_FILE_NEW);
   file->insertItem("&Open...", this, SLOT(fileOpen()),
   CTRL+Key_O, M_FILE_OPEN);
   file->insertItem("&Save", this, SLOT(fileSave()), CTRL+Key_S,
    M_FILE_SAVE);
   file->insertItem("Save &As...", this, SLOT(fileSaveAs()), 0,
    M_FILE_SAVE_AS);
   file->insertItem("Revert...", this, SLOT(fileRevert()), 0,
    M_FILE_REVERT);
   file->insertSeparator();
   file->insertItem("&Close", this, SLOT(fileClose()), CTRL+Key_W,
    M_FILE_CLOSE); 
   file->insertSeparator();
   file->insertItem("&Export Snapshot...", this, SLOT(fileExport()), 0,
    M_FILE_EXPORT);
   file->insertSeparator();
   file->insertItem("&Quit", this, SLOT(close()), CTRL+Key_Q, M_FILE_QUIT);

   for (unsigned int i = 0; i < maxRecentFiles; ++i) {
      recentFileId[i] = -1;
      QVariant v = Pref[QString("files.recent.file_%1").arg(i+1)].effective();
      if (v.isValid()) recentFiles.append(v.toString());
   }

   updateRecentFiles();

   // dialogs 
   QPopupMenu *dialogs = new QPopupMenu;
   mb->insertItem("&Dialogs", dialogs, M_DIALOGS);
   dialogs->insertItem("Surfaces...", this, SLOT(dialogsSurfaces()),
    0, M_DIALOGS_SURFACES);
   dialogs->insertItem("Groups...", this, SLOT(dialogsGroups()),
    0, M_DIALOGS_GROUP); 
   dialogs->insertItem("Token List...", this, SLOT(dialogsTokenList()),
    0, M_DIALOGS_TOKEN_LIST);
   dialogs->insertItem("In Play List...", this, SLOT(dialogsInPlay()),
    0, M_DIALOGS_INPLAY_LIST);
   dialogs->insertItem("Selected List...", this, SLOT(dialogsSelected()),
    0, M_DIALOGS_SELECTED_LIST);
   dialogs->insertItem("&Properties...", this, SLOT(dialogsProperties()),
    0, M_DIALOGS_PROP);
   dialogs->insertItem("&History...", this, SLOT(dialogsHistory()),
    0, M_DIALOGS_HISTORY);
   dialogs->insertItem("&Messages...", this, SLOT(dialogsMessage()), 
    0, M_DIALOGS_MESSAGE);
   dialogs->insertItem("&Roll Dice...", this, SLOT(dialogsRoll()), CTRL+Key_R,
    M_DIALOGS_ROLL);
   dialogs->insertSeparator();
   dialogs->insertItem("Change &Player...", this, SLOT(dialogsLogin()),
    0, M_DIALOGS_LOGIN);
   dialogs->insertItem("&Set Password...", this, SLOT(dialogsSetPassword()),
    0, M_DIALOGS_PASSWORD);
   dialogs->insertItem("New Player...", this, SLOT(dialogsNewPlayer()),
    0, M_DIALOGS_NEW_PLAYER);
   dialogs->insertSeparator();
   dialogs->insertItem("Pr&eferences...", this, SLOT(dialogsPreferences()));
   dialogs->insertSeparator();
   dialogs->insertItem("New Tab Dialog", this, SLOT(dialogsTab()));  
 
   // help
   QPopupMenu *help = new QPopupMenu;
   mb->insertItem("&Help", help, M_HELP);      
   help->insertItem("&Help", this, SLOT(helpHelp()), Key_F1, M_HELP_HELP);
   help->insertItem("&About", this, SLOT(helpAbout()), 0, M_HELP_ABOUT);
   help->insertItem("About &Qt", this, SLOT(helpAboutQt()), 0,
    M_HELP_ABOUT_QT);

   // layout for main
   QVBoxLayout *vl = new QVBoxLayout(this);
   vl->setMenuBar(mb);

   // FIXME: menu bar has some sort of size hinting problem under Win98
   // seperator beneath menu bar
   QFrame *fr = new QFrame(this);
   fr->setFrameStyle(QFrame::HLine | QFrame::Sunken);
   fr->setLineWidth(1);
   fr->setMidLineWidth(0);
   vl->addWidget(fr);

   QHBoxLayout *hl = new QHBoxLayout(vl);

   // tool menu
   QPopupMenu *toolpop = new QPopupMenu;
   
   // tool button
   toolmbn = new QPushButton(this);
   toolmbn->setFixedSize(43, 27);
   QToolTip::add(toolmbn, "Tools");
   toolmbn->setPopup(toolpop);
   hl->addWidget(toolmbn);
   
   // build the toolbox
   QGrid *toolbox = new QGrid(4, NULL, "Toolbox");
   toolbox->setMargin(1);
   toolbox->setFixedWidth(27*4+1);
   toolpop->insertItem(toolbox);
   
   // group tools
   connect(&toolgroup, SIGNAL(clicked(int)), this, SLOT(setTool(int)));

   // construct tool buttons 
   QString tooltip[TLastTool];
   tooltip[TMove] = "Move";
   tooltip[TPlot] = "Plot";
   tooltip[TSelect] = "Select";
   tooltip[TWand] = "Wand";
   tooltip[TClone] = "Clone";
   tooltip[TTrash] = "Trash";
   tooltip[TFlipLeft] = "Flip Left";
   tooltip[TFlipRight] = "Flip Right";
   tooltip[TRotateCW] = "Rotate CW";
   tooltip[TRotateCCW] = "Rotate CCW";
   tooltip[TRegroup] = "Regroup";
   tooltip[TCrayon] = "Crayon";
   tooltip[TZoomIn] = "Zoom In";
   tooltip[TZoomOut] = "Zoom Out";
   tooltip[TProp] = "Properties";
   
   char **toolpix[TLastTool];
   toolpix[TMove] = move_xpm;
   toolpix[TPlot] = plot_xpm;
   toolpix[TSelect] = select_xpm;
   toolpix[TWand] = wand_xpm;
   toolpix[TClone] = clone_xpm;
   toolpix[TTrash] = trash_xpm;
   toolpix[TFlipLeft] = flipleft_xpm;
   toolpix[TFlipRight] = flipright_xpm;
   toolpix[TRotateCW] = rotatecw_xpm;
   toolpix[TRotateCCW] = rotateccw_xpm;
   toolpix[TRegroup] = group_xpm;
   toolpix[TZoomIn] = zoomin_xpm;
   toolpix[TCrayon] = pencil_xpm;
   toolpix[TZoomOut] = zoomout_xpm;
   toolpix[TProp] = info_xpm;

   for (unsigned int i = 0; i < TLastTool-1; ++i) {
      toolbn[i] = new QPushButton(toolbox);
      toolbn[i]->setPixmap(QPixmap((const char **)toolpix[i]));
      toolbn[i]->setFixedSize(27, 27);
      QToolTip::add(toolbn[i], tooltip[i]);
      toolgroup.insert(toolbn[i], i);
   }

   hl->addStretch();

   // seperator beneath tool buttons 
   fr = new QFrame(this);
   fr->setFrameStyle(QFrame::HLine | QFrame::Sunken);
   fr->setLineWidth(1);
   fr->setMidLineWidth(0);
   vl->addWidget(fr);

   // tab dock
   tabdock = new TabHolder(this);
   vl->addWidget(tabdock);

   // create properties dialog
   propdialog = new PropertiesDialog;
   
   // track state changes
   changed = false;
   connect(this, SIGNAL(updateChanged(bool)), this, SLOT(setChanged(bool)));

   // disable inapplicable items
   setVariousEnabled(false);
   setExportEnabled(false);

   // set default angle of rotation
   setDTheta(60);

   // set default tool
   setTool(TMove);

   // create preferences dialog
   prefsdialog = new PreferencesDialog;
   connect(prefsdialog, SIGNAL(preferenceChanged(const QString)),
    this, SIGNAL(preferenceChanged(const QString))); 
 
   // set MIME types for drags
   GroupTreeItemMIMEtype = "board-" + qApp->sessionId() + "/group";
   TokenMIMEtype = "board-" + qApp->sessionId() + "/token";
   TabMIMEtype = "board-" + qApp->sessionId() + "/tab";
}


QPopupMenu *Main::findMenu(int id) const {
   return mb->findItem(id)->popup();
}


int Main::tool() const { return Tool; }


int Main::dTheta() const { return DTheta; }


QString Main::title() const { return Game->title; }


void Main::setChanged(bool ch) {
   if (!Game) return;

   QValueList<Surface *> sl = Game->sid2s.values();
   for (QValueList<Surface *>::const_iterator i = sl.begin();
    i != sl.end(); ++i) {
      QString cap = Game->title + " - " + (*i)->surfaceName() +
       (ch ? "  (Changed)" : "");
      (*i)->setCaption(cap);   
   }

   Game->changed = ch;
}


void Main::setTool(int t) { 
   QButton *b = toolgroup.find(t);
   if (b) {
      // hide menu if visible
      if (b->isVisible()) {        
         QWidget *g = b->parentWidget();
         if (g && g->isVisible()) {
            QWidget *m = g->parentWidget();
            if (m && m->inherits("QPopupMenu")) m->close();
         }      
      }
      
      // change tool button pixmap to reflect new tool
      toolmbn->setPixmap(QPixmap(*(b->pixmap())));
   }
      
   Tool = t;   
}


void Main::setToolMove() { setTool(TMove); }
void Main::setToolPlot() { setTool(TPlot); }
void Main::setToolSelect() { setTool(TSelect); }
void Main::setToolWand() { setTool(TWand); }
void Main::setToolClone() { setTool(TClone); }
void Main::setToolTrash() { setTool(TTrash); }
void Main::setToolFlipLeft() { setTool(TFlipLeft); }
void Main::setToolFlipRight() { setTool(TFlipRight); }
void Main::setToolRotateCCW() { setTool(TRotateCCW); }
void Main::setToolRotateCW() { setTool(TRotateCW); }
void Main::setToolRegroup() { setTool(TRegroup); }
void Main::setToolZoomIn() { setTool(TZoomIn); }
void Main::setToolZoomOut() { setTool(TZoomOut); }
void Main::setToolProperties() { setTool(TProp); }


void Main::setDTheta(int dt) { DTheta = dt; }


void Main::fileNew() {
   // get path to gamebox or scenario
   QString f = QFileDialog::getOpenFileName(".",
    "Gameboxes (*.box);;Scenarios (*.scn)", this, "new", "New...");
   if (f.isEmpty()) return; 
   fileNew(QFileInfo(f));
}


void Main::fileNew(const QFileInfo &file) {
   GameObj *g = new GameObj;
   BaseHandler *handler = 0;

   // determine what we're opening
   if (file.extension(FALSE) == "box") {
      g->boxfile = file;
      handler = new GameboxHandler(g, g->boxfile.fileName());
   }
   else if (file.extension(FALSE) == "scn") {
      g->scenariofile = file;
      handler = new ScenarioHandler(g, g->scenariofile.fileName());
   }
   else {
      warn("'" + file.fileName() + "' is neither a gamebox nor a scenario!");
      delete g;
      return;
   }

   if (!fileOk(file)) {
      delete g;
      delete handler;
      return;
   }
   
   if (!loadGame(g, file.filePath(), handler)) {
      delete g;
      delete handler;
      return;
   }

   delete handler;
}


void Main::fileOpen() { 
   // get path to saved game
   QString f;
   f = QFileDialog::getOpenFileName(".", "Saved games (*.sav)",
    this, "open", "Open...");
   if (f.isEmpty()) return;
   fileOpen(QFileInfo(f));
}


void Main::fileOpen(int n) {
   // get path to recent save n
   fileOpen(QFileInfo(recentFiles[n]));
}


void Main::fileOpen(const QFileInfo &file) {
   GameObj *g = new GameObj;
   g->savefile = file;

   if (!fileOk(g->savefile)) { delete g; return; }
   SaveHandler handler(g, g->savefile.fileName());
   if (!loadGame(g, file.filePath(), &handler)) { delete g; return; }
}


bool Main::loadGame(GameObj *g, const QString &file, BaseHandler *handler) {
   QXmlSimpleReader reader;
   reader.setContentHandler(handler);
   reader.setErrorHandler(handler);

   QString data;

   // fail if file cannot be read
   if (!readFile(file, data)) return false;
   
   QXmlInputSource source;
   source.setData(data);

   // fail on any parse error
   if (!reader.parse(&source)) return false;

   // new game loaded successfully, so delete old game
   delete Game;
   Game = g;

   // finish setup for the log dialog now that saved actions are recreated
   g->historydialog->finalize();

   // normalize token heights
   QValueList<Surface *> sl = g->sid2s.values();
   for (QValueList<Surface *>::const_iterator i = sl.begin();
    i != sl.end(); ++i) (*i)->normalizeTokenHeights();

   connect(&g->done, SIGNAL(completed(Action *)),
    g->messagedialog, SLOT(logAction(Action *)));
 
   connect(&g->done, SIGNAL(logged(Action *)),
    g->historydialog, SLOT(logAction(Action *)));
   connect(&g->done, SIGNAL(unlogged(Action *)),
    g->historydialog, SLOT(unlogAction(Action *)));
   connect(&g->done, SIGNAL(aborted(Action *)),
    g->historydialog, SLOT(abortAction(Action *)));

   // enable newly-applicable menu items
   setVariousEnabled(true);
   if (g->sid2s.count()) setExportEnabled(true);

   // open dialogs in tabdock
   tabdock->addPage(g->surfacesdialog);
   tabdock->addPage(g->groupsdialog);

   // login some player
   initialPlayerLogin(g);

   // add savefile to recent list
   if (!g->savefile.filePath().isEmpty())
    prependRecentFile(g->savefile.filePath());
   
   // connect changed signals from log
   connect(&g->done, SIGNAL(changed(bool)), this, SLOT(setChanged(bool)));

   emit updateChanged(false);

   return true;
}


bool Main::saveGame(GameObj *g, const QFileInfo &file) {
   if (file.filePath().isEmpty()) {
      warn("No file given for save.");
      return false;
   }
   
   // construct save XML document
   QDomDocument doc("save");

   // save
   QDomElement root = doc.createElement("save");
   root.setAttribute("gamebox", g->boxfile.filePath());
   root.setAttribute("id", g->boxid);
   root.setAttribute("version", g->boxver);
   root.setAttribute("using", BOARD_VERSION_STR);
   doc.appendChild(root);
 
   // surfaces
   QDomElement surfaces = doc.createElement("surfaces");
   root.appendChild(surfaces);

   QValueList<Surface *> sl = g->sid2s.values();   
   for (QValueList<Surface *>::const_iterator i = sl.begin();
    i != sl.end(); ++i) surfaces.appendChild((*i)->dumpXML(doc));

   // pieces: players, groups, tokens 
   root.appendChild(g->root->dumpXML(doc));

   // actions
   root.appendChild(g->done.dumpXML(doc));

   // write save file
   if (!writeXML(file, doc)) return false;

   emit updateChanged(false);
   return true;
}


void Main::initialPlayerLogin(GameObj *g) {
   QValueList<Player *> list = g->root->players();
   Player *player = 0;

   if (list.count()) {
      player = PlayerSelectDialog::getPlayer(list, this);
      if (!player) QMessageBox::information(this, "board",
       "You have not selected a player, so will be logged in as Default.",
       QMessageBox::Ok);
   }
   else QMessageBox::information(this, "board",
    "There are no players, so you will be logged in as Default.",
    QMessageBox::Ok);
   
   // create default user, if necessary
   if (!player) {
      player = new Player(g, "Default", "");
      g->groupsdialog->insert(player);
   }

   loginPlayer(g, player);
}


void Main::loginPlayer(GameObj *g, Player *player) {
   g->user = player;
   
   // FIXME: groupdialog should be updated to close invisible groups
   adjustVisibility(g, player, g->root);
}


void Main::adjustVisibility(GameObj *g, Player *p, Group *gp) {
   if (gp->permission(p, Perms::Show) && 
       (!gp->parent() || gp->parent()->isVisible())) {
      gp->show();
      g->groupsdialog->setVisibility(gp, true);
   }
   else {
      gp->hide();
      g->groupsdialog->setVisibility(gp, false);
   }

   for (Group::const_iterator i = gp->begin(); i != gp->end(); ++i)
    if (dynamic_cast<Group *>(*i)) adjustVisibility(g, p, (Group *)*i);
}


QDomDocument Main::readXML(const QFileInfo &file, const QString &doctype,
 int mode) {
   QDomDocument doc(doctype);
 
   QString data;
   if (!readFile(file, data, mode)) return QDomDocument();

   QString errorMsg;
   int errorLine, errorCol;
   if (!doc.setContent(data, &errorMsg, &errorLine, &errorCol)) {
      warn(QString("Error parsing '%1': %2, line %3, col %4.")
       .arg(file.filePath()).arg(errorMsg).arg(errorLine).arg(errorCol));
      return QDomDocument();
   }
   
   if (doc.documentElement().tagName() != doctype) {
      warn("File '" + file.filePath() + "' is not a " +
       doctype + "!");
      return QDomDocument();
   }

   return doc;  
}        


bool Main::writeXML(const QFileInfo &file, const QDomDocument &doc,
 int mode) const {
   return writeFile(file, doc.toString(), mode);
}


/*
bool Main::writeLog(GameObj *g, const QFileInfo &file) {
   if (file.filePath().isEmpty()) {
      warn("No file given for log.");
      return false;
   }

   // construct log XML document
   QDomDocument doc("log");
   QDomElement root = doc.createElement("log");
   doc.appendChild(root);

   // actions
   root.appendChild(g->done.dumpXML(doc));
  
   // write log file
   if (!writeFile(file, doc.toCString())) return false; 
 
   emit updateChanged(false);
   return true;
}
*/


void Main::fileSave() {
   if (Game->savefile.filePath().isEmpty()) fileSaveAs();
   else saveGame(Game, Game->savefile);              
}


void Main::fileSaveAs() {
   QFileInfo sf(QFileDialog::getSaveFileName(".", "Saved games (*.sav)",
    this, "saveas", "Save as..."));

   if (sf.filePath().isEmpty()) return;

   if (sf.extension(false) != "sav") sf.setFile(sf.dirPath(),
    sf.fileName() + ".sav");
   
   if (sf.exists() && !sf.isWritable()) {
      warn("Cannot write to '" + sf.filePath() + "'!");
      return;
   }

   if (!saveGame(Game, sf)) return;

   Game->savefile = QFileInfo(sf);
   prependRecentFile(Game->savefile.filePath());
}


void Main::fileRevert() {
   // FIXME: revert shouldn't reload the gamebox, just the scenario or save
   if (Game->savefile.exists()) fileOpen(Game->savefile);
   else fileNew(Game->scenariofile);
}


void Main::fileClose() {
   if (!savePrompt()) return;
   delete Game;
   Game = 0;

   // disable inapplicable menu items
   setVariousEnabled(false);
   setExportEnabled(false);
}


void Main::closeEvent(QCloseEvent *e) {
   if (!savePrompt()) {
      e->ignore();
      return;
   }

   if (!saveUserPrefs()) { 
      if (QMessageBox::warning(this, "board",
       "User preferences were not saved. Quit anyway?", QMessageBox::Ok,
       QMessageBox::Cancel) == QMessageBox::Cancel) {
         e->ignore();
         return;
      }
   }
   e->accept();
}


void Main::fileExport() {
   QStringList list; 
   bool ok = false;

   QValueList<Surface *> sl = Game->sid2s.values();
   QValueList<Surface *>::const_iterator i;

   // construce a list of surface names
   for (i = sl.begin(); i != sl.end(); ++i) list.append((*i)->surfaceName());

   QString name = QInputDialog::getItem("Export a surface", "Surface:",
    list, 0, false, &ok, this);

   if (!ok || name.isNull()) return;

   // check for a match   
   for (i = sl.begin(); i != sl.end() && (*i)->surfaceName() != name; ++i);
   Surface *s = *i;
   
   // copy canvas to a pixmap
   QPixmap pix(s->canvas()->width(), s->canvas()->height());
   QPainter p(&pix);  
   s->canvas()->drawArea(QRect(QPoint(0, 0), s->canvas()->size()), &p);
                         
   QFileInfo picfile(QFileDialog::getSaveFileName(".", 
    "PNG Image (*.png)", this, "export", "Export Snapshot..."));

   if (picfile.filePath().isEmpty()) return;

   if (picfile.exists() && !picfile.isWritable()) {
      warn("Cannot write to '" + picfile.filePath() + "'!");
      return;  
   }

   // save pixmap as a png
   if (!pix.save(picfile.filePath(), "PNG")) {
      warn("Could not save '" + picfile.filePath() + "'!");
      return;
   }
}


void Main::loadDefaultPrefs() {
   // files
   for (unsigned int i = 0; i < maxRecentFiles; ++i)
    Pref[QString("files.recent.file_%1").arg(i+1)] = Preference("default");

   Pref["files.compress"] = Preference("default", QVariant(true, 0));

   // stacking
   Pref["stacking.offset.x"] = Preference("default", -3);
   Pref["stacking.offset.y"] = Preference("default", -3);
   Pref["stacking.restack_after_invert"] = 
    Preference("default", QVariant(false, 0));

   // keys
   Pref["key.tool.move"] = Preference("default", Key_1);
   Pref["key.tool.plot"] = Preference("default", Key_2);
   Pref["key.tool.select"] = Preference("default", Key_3);
   Pref["key.tool.wand"] = Preference("default", Key_4);
   Pref["key.tool.clone"] = Preference("default", Key_Q);
   Pref["key.tool.trash"] = Preference("default", Key_W);
   Pref["key.tool.flip_left"] = Preference("default", Key_E);
   Pref["key.tool.flip_right"] = Preference("default", Key_R);
   Pref["key.tool.rotate_ccw"] = Preference("default", Key_A);
   Pref["key.tool.rotate_cw"] = Preference("default", Key_S);
   Pref["key.tool.regroup"] = Preference("default", Key_D);
   Pref["key.tool.zoom_in"] = Preference("default", Key_Z);
   Pref["key.tool.zoom_out"] = Preference("default", Key_X);
   Pref["key.tool.properties"] = Preference("default", Key_C);
}
        

bool Main::loadUserPrefs() {
   // fail out and use defaults if user prefs are unavailable
#ifdef Q_WS_WIN
   preffile = QDir::currentDirPath() + "/user.xml";
#else
   preffile = QDir::homeDirPath() + "/.board/user.xml";
#endif 

   // check whether user.xml exists; otherwise don't bother
   if (!preffile.exists()) return false;

   QDomDocument pdoc = readXML(preffile, "preferences");
   if (pdoc.isNull()) return false;

   QDomNodeList l = pdoc.documentElement().childNodes();
   for (unsigned int i = 0, c = l.count(); i < c; i++)
    traverseUserPrefsTree(l.item(i));

   return true;
}


void Main::traverseUserPrefsTree(QDomNode node) {
   QDomNodeList l = node.childNodes();
   for (unsigned int i = 0, c = l.count(); i < c; i++)
    traverseUserPrefsTree(l.item(i));
   
   if (!node.isNull()) {
      QDomElement e = node.toElement();
      QDomNamedNodeMap attr = e.attributes();
      for (unsigned int i = 0, c = attr.length(); i < c; i++) {
         QDomNode a = attr.item(i);
         QString key = a.nodeName();
         QDomNode n = e;
         while (n.nodeName() != "preferences") {
            key = n.nodeName() + "." + key;
            n = n.parentNode();
         }

         // FIXME: do some sort of error checking here, otherwise
         // we could end up with invalid preferences
         if (Pref.contains(key)) {
            switch(Pref[key]["default"].type()) {
               case QVariant::Int:
                  Pref[key]["user"] = a.nodeValue().toInt();
                  break;
               case QVariant::Bool:
                  Pref[key].insert(QString("user"),
                   QVariant(a.nodeValue() == "true" ? true : false, 0));
                  break;
               case QVariant::String:
               default:
                  Pref[key]["user"] = a.nodeValue();
                  break;
            }
         }
      }
   }
}
        

bool Main::saveUserPrefs() {
   // ensure we have someplace to write the preferences   
   QDir dir(preffile.dir());
   if (!dir.exists()) {
      if (!dir.mkdir(dir.path())) {
         warn("Cannot create '" + dir.path() + "'!");
         return false;
      }
   }

   // ensure that the preferences file is writable
   if (preffile.exists() && !preffile.isWritable()) {
      warn("Cannot write to '" + preffile.filePath() + "'!");
      return false;
   }

   // construct XML document
   QDomDocument doc("preferences");
   QDomElement root = doc.createElement("preferences");
   doc.appendChild(root);
     
   // iterate over preferences
   for (PreferenceMap::ConstIterator i = Pref.begin(); i != Pref.end(); ++i) {
      // skip default or empty preferences
      if ((*i).level() == Preference::Default ||
       !(*i).effective().isValid()) continue;

      QDomElement e = root;
      QString k = i.key();

      int pos;
      while ((pos = k.find('.')) != -1) {
         QString name = k.left(pos);
         
         QDomNode a = e.firstChild();
         while (!a.isNull() && a.nodeName() != name) a = a.nextSibling();
       
         if (a.isNull()) {  
            QDomElement n = doc.createElement(name);
            e.appendChild(n);
            e = n;
         }
         else e = a.toElement();
  
         k = k.mid(pos+1);
      }      
      
      switch ((*i)["user"].type()) {
         case QVariant::Bool:
            e.setAttribute(k, (*i)["user"].toBool() ? "true" : "false");
            break;
         default:
            e.setAttribute(k, (*i)["user"].toString());
      }
   } 

   // preferences of the world, write!
   if (!writeXML(preffile, doc, Uncompressed)) return false;

   return true;   
}


void Main::prependRecentFile(const QString &filename) {
   // prepend a new file to the recent file list
   recentFiles.remove(filename);
   recentFiles.push_front(filename);
   while (recentFiles.size() > maxRecentFiles) recentFiles.pop_back();

   updateRecentFiles();
}


void Main::updateRecentFiles() { 
   // adjust file menu
   QPopupMenu *file = findMenu(M_FILE);
   for (unsigned int i = 0; i < recentFiles.size(); i++) {
      // don't show the whole path; that makes the menu overly wide
      QString item = QString("&%1 " +
       QFileInfo(recentFiles[i]).fileName()).arg(i+1);

      if (recentFileId[i] == -1) {
         if (i == 0) file->insertSeparator(file->count() - 2);
         recentFileId[i] = file->insertItem(item, this,
          SLOT(fileOpen(int)), 0, -1, file->count() - 2);
         file->setItemParameter(recentFileId[i], i);
      } 
      else file->changeItem(recentFileId[i], item);

      Pref[QString("files.recent.file_%1").arg(i+1)]["user"] = recentFiles[i];
   }
}


bool Main::savePrompt() {
   // prompt for save if there is an open game 
   if (Game) {
      if (Game->changed) {
         QString f = Game->savefile.filePath().isEmpty() ? QString("newgame") :
          Game->savefile.filePath();

         int result = QMessageBox::warning(0, "Question",
          "Save changes to " + f + "?",
          QMessageBox::Yes, QMessageBox::No,
          QMessageBox::Cancel | QMessageBox::Default);

          switch (result) {
            case QMessageBox::Yes: 
               fileSave();
               if (!Game->changed) break;
            case QMessageBox::Cancel:
               return false;
            case QMessageBox::No: 
               break;
         }
      }    
   }

   return true;
}


void Main::dialogsHistory() { spawnTabDialog(Game->historydialog); }


void Main::dialogsSurfaces() { spawnTabDialog(Game->surfacesdialog); }


void Main::dialogsGroups() { spawnTabDialog(Game->groupsdialog); }


void Main::dialogsTokenList() { spawnTabDialog(Game->alldialog); }


void Main::dialogsInPlay() { spawnTabDialog(Game->inplaydialog); }


void Main::dialogsSelected() { spawnTabDialog(Game->selecteddialog); }


void Main::dialogsProperties() { /* spawnTabDialog(propdialog); */ } 


void Main::dialogsMessage() { spawnTabDialog(Game->messagedialog); }


void Main::dialogsRoll() {
   //RollDialog *rd = new RollDialog;
   //rd->show();
}


void Main::dialogsLogin() {   
   Player *player = PlayerSelectDialog::getPlayer(Game->root->players(), this);
   if (!player || player == Game->user) return;   

   loginPlayer(Game, player);
}


void Main::dialogsSetPassword() {
   QString password = PlayerSetPasswordDialog::getPassword(this);
   if (password.isNull()) return;
   else Game->user->setPassword(password);
}


void Main::dialogsNewPlayer() {
   // get player name
   bool ok = false;
   QString name = QInputDialog::getText("Create a Player", "Player Name:",
    QLineEdit::Normal, QString::null, &ok, this);
   if (!ok) return;

   // take action
   PlayerCreateAction *a = new PlayerCreateAction(name);
   if (a->isPermissible()) {
      a->commit();
      Game->done.log(a);
      Game->done.complete(); 
   }
   else delete a;
}


void Main::dialogsPreferences() { prefsdialog->show(); }


void Main::dialogsTab() { spawnTabDialog(0); }


void Main::spawnTabDialog(TabPage *page) {
   TabDialog *t = new TabDialog(0, 0, WDestructiveClose);
   if (page) t->addPage(page);
   t->show();
}


void Main::helpHelp() {
   // FIXME: write help
}


void Main::helpAbout() {
   QString s("Board version %1\nby Joel Uckelman (uckelman@nomic.net)");
   QMessageBox::about(this, "About Board", s.arg(BOARD_VERSION_STR));
}


void Main::helpAboutQt() { QMessageBox::aboutQt(this, "About Qt"); }


void Main::editUndo() { Game->historydialog->prev(); }


void Main::editRedo() { Game->historydialog->next(); }
        

void Main::setVariousEnabled(bool enable) {
   findMenu(M_FILE)->setItemEnabled(M_FILE_SAVE, enable);
   findMenu(M_FILE)->setItemEnabled(M_FILE_SAVE_AS, enable);
   findMenu(M_FILE)->setItemEnabled(M_FILE_REVERT, enable);
   findMenu(M_FILE)->setItemEnabled(M_FILE_CLOSE, enable);
   findMenu(M_DIALOGS)->setItemEnabled(M_DIALOGS_HISTORY, enable);
   findMenu(M_DIALOGS)->setItemEnabled(M_DIALOGS_SURFACES, enable);
   findMenu(M_DIALOGS)->setItemEnabled(M_DIALOGS_GROUP, enable);
   findMenu(M_DIALOGS)->setItemEnabled(M_DIALOGS_TOKEN_LIST, enable);
   findMenu(M_DIALOGS)->setItemEnabled(M_DIALOGS_INPLAY_LIST, enable);
   findMenu(M_DIALOGS)->setItemEnabled(M_DIALOGS_SELECTED_LIST, enable);
   findMenu(M_DIALOGS)->setItemEnabled(M_DIALOGS_MESSAGE, enable);
   findMenu(M_DIALOGS)->setItemEnabled(M_DIALOGS_LOGIN, enable);
   findMenu(M_DIALOGS)->setItemEnabled(M_DIALOGS_PASSWORD, enable);
   findMenu(M_DIALOGS)->setItemEnabled(M_DIALOGS_NEW_PLAYER, enable);
}


void Main::setExportEnabled(bool enable) {
   findMenu(M_FILE)->setItemEnabled(M_FILE_EXPORT, enable);
}


bool Main::fileOk(const QFileInfo &file) {
   // check whether file exists
   if (!file.exists()) {
      warn("File '" + file.filePath() + "' does not exist!");
      return false;
   }
 
   // check whether file is readable
   if (!file.isReadable()) {
      warn("File '" + file.filePath() + "' is not readable by this user!");
      return false;
   }

   return true; 
}


bool Main::readFile(const QFileInfo &file, QString &data, int mode) {
   QFile i(file.filePath());
   if (!i.open(IO_ReadOnly)) {
      warn("Cannot open file '" + file.filePath() + "'!");
      return false;
   }

   if (mode == Heuristic) {
      // Check whether the first 10 bytes of the file are "<!DOCTYPE ".
      // The probability that a *compressed* save file would have these
      // as the first 10 bytes is vanishingly small.
      QString test;
      QTextStream in(&i);
      in >> test;
      mode = test == "<!DOCTYPE" ? Uncompressed : Compressed;
      i.reset();
   }

   if (mode == Compressed) {
      QDataStream in(qUncompress(i.readAll()), IO_ReadOnly);
      in >> data;     
 
      if (data.isNull()) {
         warn("Decompression of data from file '" + file.filePath() +
          "' failed!");
         return false;
      }
   }
   else { 
      QTextStream in(&i);
      data = in.read();
   }

   i.close();

   if (i.status() != IO_Ok) {
      warn("Error reading from file '" + file.filePath() + "'!");
      return false;
   }

   return true;
}


bool Main::writeFile(const QFileInfo &file, const QString &data,
 int mode) const {
   QFile o(file.filePath());
   if (!o.open(IO_WriteOnly)) {
      warn("Cannot open file '" + file.filePath() + "'!");
      return false;
   }

   if (mode == Heuristic) {
      // check prefrences
      mode = Pref["files.compress"].effective().toBool() ?
       Compressed : Uncompressed;
   }

   if (mode == Compressed) {
      QByteArray bytes;
      QDataStream out(bytes, IO_WriteOnly);
      out << data;
      o.writeBlock(qCompress(bytes));
   }
   else {
      QTextStream out(&o);
      out << data;
   }

   o.close();

   if (o.status() != IO_Ok) {
      warn("Error writing to file '" + file.filePath() + "'!");
      return false;
   }

   return true;
}


void Main::inform(const QString &message) const {
   QMessageBox::information(MW, "board", message, QMessageBox::Ok, 0);
   cerr << "Notice: " << message << endl;
}


void Main::warn(const QString &message) const {
   QMessageBox::warning(MW, "board", message, QMessageBox::Ok, 0);
   cerr << "Warning: " << message << endl;
}


void Main::die(const QString &message) const {
   QMessageBox::critical(MW, "board", message, QMessageBox::Ok, 0);
   cerr << "Fatal: " << message << endl;
}
