// $Id: main.h,v 1.108 2004/09/01 20:47:01 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2002, 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __MAIN_H_
#define __MAIN_H_

#include <qbuttongroup.h>
#include <qdom.h>
#include <qfileinfo.h>
//#include <qmainwindow.h>
#include <qmenubar.h>
#include <qpushbutton.h>
#include <qstringlist.h>


class BaseHandler;
class GameboxHandler;
class SetupHandler;

class ComboAction;
class GameObj;
class Group;
class Player;
class PreferencesDialog;
class PropertiesDialog;
class TabHolder;
class TabPage;


//class Main : public QMainWindow {
class Main : public QWidget {
   Q_OBJECT

   friend class GameboxHandler;
   friend class SetupHandler;

   public:
      Main(QWidget *parent = 0, const char *name = 0,
       WFlags f = WType_TopLevel);
      QPopupMenu *findMenu(int id) const; 
      int tool() const;
      int dTheta() const;
      QString title() const;      

      // message functions
      void inform(const QString &message) const;
      void warn(const QString &message) const;
      void die(const QString &message) const;

   public slots:
      void setChanged(bool ch);
      void setDTheta(int dt);
     
      void setTool(int t);
      void setToolMove();
      void setToolPlot();
      void setToolSelect();
      void setToolWand();
      void setToolClone();
      void setToolTrash();
      void setToolFlipLeft();
      void setToolFlipRight();
      void setToolRotateCCW();
      void setToolRotateCW();
      void setToolRegroup();
      void setToolZoomIn();
      void setToolZoomOut();
      void setToolProperties();
      
   signals:
      void updateChanged(bool ch);
      void preferenceChanged(const QString);     
 
   private slots:
      void fileNew();
      void fileOpen();
      void fileOpen(int n);
      void fileSave();
      void fileSaveAs();
      void fileRevert();
      void fileClose();
      void fileExport();
      void dialogsHistory();
      void dialogsSurfaces();
      void dialogsGroups();
      void dialogsTokenList();
      void dialogsInPlay();
      void dialogsSelected();
      void dialogsProperties();
      void dialogsMessage();
      void dialogsRoll();
      void dialogsLogin();
      void dialogsSetPassword();
      void dialogsNewPlayer();
      void dialogsPreferences();
      void dialogsTab();
      void helpHelp();
      void helpAbout();
      void helpAboutQt();
      void editUndo();
      void editRedo();
      
   protected:
      void closeEvent(QCloseEvent *e);
      
   private:
      void fileNew(const QFileInfo &file);
      void fileOpen(const QFileInfo &file);

      bool loadGame(GameObj *g, const QString &file, BaseHandler *handler); 
      //bool writeLog(GameObj *g, const QFileInfo &file);

      // GameObj -> QDomDocument
      bool saveGame(GameObj *g, const QFileInfo &file); 

      enum { Heuristic = -1, Uncompressed, Compressed };

      // QDomDocument <-> QString
      QDomDocument readXML(const QFileInfo &file, const QString &doctype,
       int mode = Heuristic);
      bool writeXML(const QFileInfo &file, const QDomDocument &doc, 
       int mode = Heuristic) const;

      // QString <-> File
      bool readFile(const QFileInfo &file, QString &data,
       int mode = Heuristic);
      bool writeFile(const QFileInfo &file, const QString &data, 
       int mode = Heuristic) const;

      void loadDefaultPrefs(); 
      bool loadUserPrefs();
      bool saveUserPrefs();
      void traverseUserPrefsTree(QDomNode node);
     
      bool savePrompt();

      void initialPlayerLogin(GameObj *g);
      void loginPlayer(GameObj *g, Player *player);      
      void adjustVisibility(GameObj *g, Player *p, Group *gp);

      void prependRecentFile(const QString &filename);
      void updateRecentFiles();
      
      void setVariousEnabled(bool enable);
      void setExportEnabled(bool enable);

      void spawnTabDialog(TabPage *page);

      bool fileOk(const QFileInfo &file);

      enum { maxRecentFiles = 5 };
      int recentFileId[maxRecentFiles];
      QStringList recentFiles;
 
      QMenuBar *mb;
      QFileInfo preffile;
      bool changed;
      QPushButton *toolmbn, *toolbn[12], *tlistbn[5], *glistbn[5];
      QButtonGroup toolgroup;
      TabHolder *tabdock;
 
      PropertiesDialog *propdialog;
      PreferencesDialog *prefsdialog;

      int Tool, DTheta;
    
      enum { BNew, BUp, BDown, BDup, BTrash };
};

#endif /* __MAIN_H_ */
