// $Id: player.cpp,v 1.5 2004/09/21 15:13:09 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "game.h"
#include "player.h"
#include "util.h"


//
// Player
//
Player::Player(GameObj *g, const QString &n, const QString &pw, unsigned int i)
 : Group(g, n, g->root, i), Pass(pw) {
}


Player *Player::owner() { return this; }


QDomElement Player::dumpXML(QDomDocument &doc) const {
   QDomElement e = doc.createElement("player");
   e.setAttribute("name", Name);
   e.setAttribute("password", Pass);
   e.setAttribute("id", id());

   QValueList<Player *> plist = game->root->players();
   for (QValueList<Player *>::const_iterator i = plist.begin();
    i != plist.end(); ++i) {
      // state
      QDomElement s = doc.createElement("state");
      s.setAttribute("pid", (*i)->id());

      // permissions
      if (perms.contains(*i))
       s.setAttribute("permissions", perms.toString(*i));
      e.appendChild(s);

      // drawing
      QDomElement d = doc.createElement("drawing");
      d.setAttribute("pid", (*i)->id());
      d.setAttribute("width", drawingWidth(*i));
      d.setAttribute("margin", drawingMargin(*i));
      d.setAttribute("color", rgb2str(drawingColor(*i).rgb()));

      e.appendChild(d);
   }

   // children
   for (const_iterator i = begin(); i != end(); ++i)
    e.appendChild((*i)->dumpXML(doc));
   return e;
}


QString Player::password() const { return Pass; }


void Player::setPassword(const QString &p) { Pass = p; }


QColor Player::drawingColor(Player *p) const {
   return pref[p].drawing.color;
}


void Player::setDrawingColor(Player *p, const QColor &c) {
   pref[p].drawing.color = c;
}


unsigned int Player::drawingWidth(Player *p) const {
   return pref[p].drawing.width;
}


void Player::setDrawingWidth(Player *p, unsigned int w) {
   pref[p].drawing.width = w;
}


unsigned int Player::drawingMargin(Player *p) const {
   return pref[p].drawing.margin;
}


void Player::setDrawingMargin(Player *p, unsigned int m) {
   pref[p].drawing.margin = m; }
