// $Id: playerdialogs.cpp,v 1.4 2004/04/11 05:14:33 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <qlayout.h>

#include "md5hash.h"
#include "playerdialogs.h"


//
// PlayerSelectDialog
//
PlayerSelectDialog::PlayerSelectDialog(const QValueList<Player *> &l,
 QWidget *parent, const char *name, bool modal, WFlags f) :
 QDialog(parent, name, modal, f), list(l) {
   setCaption("Select a Player");
   
   QVBoxLayout *vb = new QVBoxLayout(this, 5);

   // first and second rows
   QHBoxLayout *hb = new QHBoxLayout(vb);
   QVBoxLayout *col1 = new QVBoxLayout(hb);
   QVBoxLayout *col2 = new QVBoxLayout(hb);

   // drop box   
   combo = new QComboBox(false, this);
   for (QValueList<Player *>::const_iterator i = list.begin();
    i != list.end(); ++i) combo->insertItem((*i)->name());
   col2->addWidget(combo);

   // label for drop box
   QLabel *label = new QLabel(combo, "Player:", this);   
   col1->addWidget(label);

   // line edit
   le = new QLineEdit(this);
   le->setEchoMode(QLineEdit::Password);
   le->setFocus();
   col2->addWidget(le);
   
   // label for line edit
   label = new QLabel(le, "Password:", this);   
   col1->addWidget(label);

   // third row
   hb = new QHBoxLayout(vb);
   hb->addStretch();
   
   message = new QLabel(QString::null, this);
   message->setAlignment(AlignCenter | ExpandTabs | ShowPrefix);
   QPalette pal = message->palette();
   pal.setColor(QColorGroup::Foreground, QColor("red"));
   message->setPalette(pal);
   hb->addWidget(message);
   
   hb->addStretch();

   // fourth row
   hb = new QHBoxLayout(vb);
   hb->insertStretch(0);   

   // buttons
   QPushButton *ok = new QPushButton("&Ok", this);
   connect(ok, SIGNAL(clicked()), this, SLOT(checkPassword()));
   ok->setDefault(true);
   hb->addWidget(ok); 

   QPushButton *cancel = new QPushButton("&Cancel", this);
   connect(cancel, SIGNAL(clicked()), this, SLOT(reject()));
   hb->addWidget(cancel);
};


void PlayerSelectDialog::checkPassword() {
   QString passwd = list[combo->currentItem()]->password();
   if (le->text().isEmpty() && passwd.isEmpty() ||
    md5hash(le->text()) == passwd) accept();
   else {
      le->clear();
      if (message->text().isNull()) message->setText("Incorrect password!");
   }
}


Player *PlayerSelectDialog::getPlayer(const QValueList<Player *> &pl,
 QWidget *parent) {
   PlayerSelectDialog sd(pl, parent);
   Player *player = sd.exec() == Accepted ?
    sd.list[sd.combo->currentItem()] : 0;
   return player;
}


//
// PlayerSetPasswordDialog
//
PlayerSetPasswordDialog::PlayerSetPasswordDialog(QWidget *parent,
 const char *name, bool modal, WFlags f) : QDialog(parent, name, modal, f) {
   setCaption("Set Player Password");

   QVBoxLayout *vb = new QVBoxLayout(this, 5);

   // first and second rows
   QHBoxLayout *hb = new QHBoxLayout(vb);
   QVBoxLayout *lcol = new QVBoxLayout(hb);
   QVBoxLayout *rcol = new QVBoxLayout(hb);

   // password edit
   pass = new QLineEdit(this);
   pass->setEchoMode(QLineEdit::Password);
   pass->setFocus();
   rcol->addWidget(pass);

   // password edit label
   QLabel *plabel = new QLabel(pass, "Password:", this);
   lcol->addWidget(plabel);

   // confirmation edit
   conf = new QLineEdit(this);
   conf->setEchoMode(QLineEdit::Password);
   rcol->addWidget(conf);

   // confirmation edit label
   QLabel *clabel = new QLabel(conf, "Confirm:", this);
   lcol->addWidget(clabel);

   // third row
   hb = new QHBoxLayout(vb);
   hb->addStretch();

   // buttons
   ok = new QPushButton("&Ok", this);
   ok->setFocusPolicy(QWidget::ClickFocus);
   hb->addWidget(ok);

   QPushButton *cancel = new QPushButton("&Cancel", this);
   hb->addWidget(cancel);

   connect(ok, SIGNAL(clicked()), this, SLOT(activate()));
   connect(cancel, SIGNAL(clicked()), this, SLOT(reject()));
}


void PlayerSetPasswordDialog::activate() {
   // move to confirm if focused on password
   if (pass->hasFocus()) conf->setFocus();
   // else check for match
   else if (pass->text() == conf->text()) accept();
   // else fail and clear the fields
   else {   
      pass->clear();
      conf->clear();
      pass->setFocus();
   }
}


QString PlayerSetPasswordDialog::getPassword(QWidget *parent) {
   QString password;
   PlayerSetPasswordDialog pd(parent);
   if (pd.exec() == Accepted) {
      if (pd.pass->text().isEmpty()) return QString("");
      else return md5hash(pd.pass->text());
   }
   else return QString();
}

