// $Id: playerprop.cpp,v 1.3 2004/08/12 22:06:03 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <qcolordialog.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qgroupbox.h>

#include "game.h"
#include "player.h"
#include "playerprop.h"


//
// PlayerPropertiesDialog
//
PlayerPropertiesDialog::PlayerPropertiesDialog(GameObj *g, Player *p,
 QWidget *parent, const char *name, bool modal, WFlags f) :
 QDialog(parent, name, modal, f), game(g), player(p) {
   setCaption(player->name() + " - Properties");

   Player *user = game->user;

   QVBoxLayout *vb = new QVBoxLayout(this, 2);

   // name
   QHBoxLayout *hb = new QHBoxLayout(vb);
   playerName = new QLineEdit(player->name(), this);
   QLabel *nl = new QLabel(playerName, "Name:", this);
   hb->addWidget(nl);
   hb->addWidget(playerName);

   hb = new QHBoxLayout(vb);

   // drawing group
   QGroupBox *gb = new QGroupBox("Drawing", this);
   // NOTE: setColumnLayout with 0 columns makes the QGroupBox create the
   // layout for the frame and title, but not for the contents.
   gb->setColumnLayout(0, Qt::Vertical);
   hb->addWidget(gb);
   
   QGridLayout *grid = new QGridLayout(gb->layout(), 3, 2, 5);

   colorButton = new QPushButton("", gb);
   colorButton->setFixedSize(27, 27);
   setColorButton(user->drawingColor(player));
   connect(colorButton, SIGNAL(clicked()), this, SLOT(setDrawingColor()));
   QLabel *cl = new QLabel(colorButton, "Color:", gb);
   grid->addWidget(cl, 0, 0);
   grid->addWidget(colorButton, 0, 1);

   lineWidth = new QSpinBox(0, 20, 1, gb);
   lineWidth->setValue(user->drawingWidth(player));
   lineWidth->setFixedWidth(lineWidth->fontMetrics().width("000000"));
   QLabel *wl = new QLabel(lineWidth, "Width:", gb);
   grid->addWidget(wl, 1, 0);
   grid->addWidget(lineWidth, 1, 1);
 
   marginWidth = new QSpinBox(0, 20, 1, gb);
   marginWidth->setValue(user->drawingMargin(player));
   marginWidth->setFixedWidth(marginWidth->fontMetrics().width("000000"));
   QLabel *ml = new QLabel(marginWidth, "Margin:", gb);
   grid->addWidget(ml, 2, 0);
   grid->addWidget(marginWidth, 2, 1);
   
   hb->insertStretch(-1);
   vb->insertStretch(-1);

   // buttons
   hb = new QHBoxLayout(vb);
   hb->insertStretch(0);

   QPushButton *ok = new QPushButton("&Ok", this);
   connect(ok, SIGNAL(clicked()), this, SLOT(accept()));
   ok->setDefault(true);
   hb->addWidget(ok);

   QPushButton *cancel = new QPushButton("&Cancel", this);
   connect(cancel, SIGNAL(clicked()), this, SLOT(reject()));
   hb->addWidget(cancel);
}


bool PlayerPropertiesDialog::setProperties(GameObj *game, Player *player,
 QWidget *parent) {
   PlayerPropertiesDialog ppd(game, player, parent);
   if (ppd.exec() == Accepted) {
      // FIXME: should there be an Action here?

      Player *user = game->user;

      QString name = ppd.playerName->text();
      unsigned int width = ppd.lineWidth->value();
      unsigned int margin = ppd.marginWidth->value();

      bool changed = false;

      if (name != player->name()) {
         player->setName(name);
         changed = true;
      }

      if (ppd.color != user->drawingColor(player)) {
         user->setDrawingColor(player, ppd.color);
         changed = true;
      }

      if (width != user->drawingWidth(player)) {
         user->setDrawingWidth(player, width);
         changed = true;
      }

      if (margin != user->drawingMargin(player)) {
         user->setDrawingMargin(player, margin);
         changed = true;
      }
      
      return changed;
   }
   else return false;
}


void PlayerPropertiesDialog::setDrawingColor() {
   QColor c = QColorDialog::getColor(color);
   if (c.isValid() && c != color) setColorButton(c); 
}


void PlayerPropertiesDialog::setColorButton(const QColor &c) {
   QPixmap p(17, 17);
   p.fill(c);
   colorButton->setIconSet(QIconSet(p));
   color = c;
}
