// $Id: polyline.h,v 1.1 2004/09/21 15:03:14 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2002, 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __POLYLINE_H_
#define __POLYLINE_H_

#include <qcanvas.h>
#include <qvaluestack.h>

#include "group.h"


class Polyline; 


class PolylineSegment : public QCanvasLine {
   public:
      PolylineSegment(QPoint p1, QPoint p2, QCanvas *c, Polyline *s);
      virtual int rtti() const;
      static const int RTTI = 500001;

   private:
      Polyline *pline;
};


class Polyline : public Groupable {
   public:
      Polyline(GameObj *g, const QString &n, Group *gr, QCanvas *c,
       unsigned int i);
      virtual ~Polyline();
      void pushPoint(QPoint p2);
      QPoint popPoint();
      void setPen(QPen p);
      virtual void show();
      virtual void hide();
      virtual bool isVisible() const;
      QCanvas *canvas() const;
      void setCanvas(QCanvas *c);
      void setX(double x);
      void setY(double y);
      void setZ(double z);
      virtual bool permission(const Player *pl, unsigned int pm) const;
      virtual QDomElement dumpXML(QDomDocument &doc) const;

   private:
      QValueStack<PolylineSegment *> segments;
      QCanvas *cv;
      QCanvasEllipse *dot;
      QPen pen;
};


#endif /* __CRAYON_H_ */
