// $Id: preferencesdialog.cpp,v 1.7 2004/04/03 17:45:32 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <qinputdialog.h>
#include <qlayout.h>
#include <qlistview.h>

#include "preferences.h"
#include "preferencesdialog.h"


extern PreferenceMap Pref; 


//
// PreferencesDialog
//
PreferencesDialog::PreferencesDialog(QWidget *parent, const char *name,
 WFlags f) : QWidget(parent, name, f), lv(this) {
   QVBoxLayout *vb = new QVBoxLayout(this);

   vb->addWidget(&lv);

   lv.addColumn("Preference Name");
   lv.addColumn("Status");
   lv.addColumn("Type");
   lv.addColumn("Value");

   lv.setAllColumnsShowFocus(true);
   lv.setShowSortIndicator(true);

   for (PreferenceMap::ConstIterator i = Pref.begin(); i != Pref.end(); ++i)
    new QListViewItem(&lv, i.key(), levelToString((*i).level()),
    typeToString((*i).effective().type()), (*i).effective().toString());

   connect(&lv, SIGNAL(doubleClicked(QListViewItem *)),
    this, SLOT(editItem(QListViewItem *)));
   
   connect(this, SIGNAL(preferenceChanged(const QString)),
    this, SLOT(updatePreference(const QString)));
}


void PreferencesDialog::editItem(QListViewItem *i) {
   bool ok;

   if (i->text(2) == "integer") {
      int val = QInputDialog::getInteger("Enter an integer value", i->text(0),
       i->text(3).toInt(), -2147483647, 2147483647, 1, &ok, this);
      if (ok) Pref[i->text(0)]["user"] = val;
   }
   else if (i->text(2) == "boolean") {
      QString val = QInputDialog::getText("Enter a boolean value", i->text(0),
       QLineEdit::Normal, i->text(3) ? "true" : "false", &ok, this);
      if (ok && (val == "true" || val == "false"))
       Pref[i->text(0)].insert(QString("user"),
       QVariant(val == "true" ? true : false, 0));
   }
   else if (i->text(2) == "string") {
      QString val = QInputDialog::getText("Enter a string value", i->text(0),
       QLineEdit::Normal, i->text(3), &ok, this);
      if (ok) Pref[i->text(0)]["user"] = val;
   }

   if (ok) emit preferenceChanged(i->text(0));
}


void PreferencesDialog::updatePreference(const QString key) {
   QListViewItem *i = lv.findItem(key, 0);
   if (i) {
      i->setText(1, levelToString(Pref[key].level()));
      i->setText(2, typeToString(Pref[key].effective().type()));
      i->setText(3, Pref[key].effective().toString());
   }
}


QString PreferencesDialog::typeToString(int type) {
   switch (type) {
      case QVariant::Bool:    return "boolean"; 
      case QVariant::Int:     return "integer";
      case QVariant::String:  return "string";
      case QVariant::Invalid: return "invalid";
      default:                return "bug"; 
   }
}
 

QString PreferencesDialog::levelToString(int level) {
   switch (level) {
      case Preference::Default: return "default";
      case Preference::User:    return "user";
      case Preference::Game:    return "game";
      default:                  return "bug";
   }
}
