// $Id: roll.cpp,v 1.17 2004/07/29 16:47:25 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2002, 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <qcheckbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qmultilineedit.h>
#include <qpushbutton.h>
#include <qvalidator.h>
#include <stdlib.h>

#include "action.h"
#include "game.h"
#include "roll.h"

extern GameObj *Game;


//
// RollDialog
// 
RollDialog::RollDialog(QWidget *parent, const char *name, WFlags f) :
 QWidget(parent, name, f) {

   setCaption("Roll Dice");

   // layout for the dialog
   QVBoxLayout *vb = new QVBoxLayout(this, 5);
   vb->setMargin(6);

   //   
   // "Roll x sets of the following:"
   //
   QHBoxLayout *hb = new QHBoxLayout(vb);
   
   QLabel *l = new QLabel("Roll", this);
   hb->addWidget(l);
   
   sets = new QLineEdit("1", this);
   sets->setMaxLength(4);
   int w_max = sets->fontMetrics().maxWidth()*3; 
   sets->setFixedWidth(w_max);
   sets->setAlignment(Qt::AlignRight);
   RValidator *v = new RValidator(1, 1000, sets);
   sets->setValidator(v);
   connect(sets, SIGNAL(textChanged(const QString &)),
    this, SLOT(validateRoll()));
   hb->addWidget(sets);
   
   l = new QLabel("sets of the following:", this);
   hb->addWidget(l);

   hb->insertStretch(-1);

   //
   // "x Roll nds"
   //
   for (int x = 0; x < 3; x++) {
      hb = new QHBoxLayout(vb);   

      hb->insertSpacing(0, l->fontMetrics().width("Roll"));
      
      cb[x] = new QCheckBox("Roll", this);
      connect(cb[x], SIGNAL(clicked()), this, SLOT(validateRoll()));
      hb->addWidget(cb[x]);
      
      dice[x] = new QLineEdit("1", this);
      dice[x]->setMaxLength(4);
      dice[x]->setFixedWidth(w_max);
      dice[x]->setAlignment(Qt::AlignRight);
      dice[x]->setValidator(v);
      connect(dice[x], SIGNAL(textChanged(const QString &)),
       this, SLOT(validateRoll()));
      hb->addWidget(dice[x]);
      
      QLabel *d = new QLabel("d", this);
      hb->addWidget(d);
      
      sides[x] = new QLineEdit("6", this);
      sides[x]->setMaxLength(4);
      sides[x]->setFixedWidth(w_max);
      sides[x]->setAlignment(Qt::AlignRight);
      sides[x]->setValidator(v);
      connect(sides[x], SIGNAL(textChanged(const QString &)),
       this, SLOT(validateRoll()));
      hb->addWidget(sides[x]);

      hb->insertSpacing(-1, 20);
      hb->insertStretch(-1);

      // insert buttons at ends of first and second rows
      if (x == 0) {
         roll = new QPushButton("Roll", this);
         connect(roll, SIGNAL(clicked()), this, SLOT(rollDice()));
         roll->setDefault(true);
         hb->addWidget(roll);
      }
      else if (x == 1) {     
         cancel = new QPushButton("Cancel", this);
         connect(cancel, SIGNAL(clicked()), this, SLOT(close()));
         hb->addWidget(cancel);
      }
   }
  
   cb[0]->setChecked(true);
 
   //
   // "Results:"
   //    
   l = new QLabel("Results:", this);
   vb->addWidget(l);
   
   display = new QMultiLineEdit(this);
   display->setReadOnly(true);
   display->setFocusPolicy(QWidget::NoFocus);  // for proper tab order
   vb->addWidget(display);
}


void RollDialog::rollDice() {  
   QString r, t;
   
   int s = sets->text().toInt();

   r.sprintf("Dice roll: %i set%s of", s, s > 1 ? "s" : "");

   for (int x = 0; x < 3; x++) {
      if (cb[x]->isChecked()) {
         r += t.sprintf(" %id%i,", dice[x]->text().toInt(),
          sides[x]->text().toInt());
      }
   }
   
   
   QString result;

   result += r;   
   display->append(r);
   emit rollResult(r);
   
   for (int x = 0; x < s; x++) {
      r = t.sprintf("%i: ", x+1);
   
      for (int y = 0; y < 3; y++) {
         if (cb[y]->isChecked()) {
            for (int z = 0; z < dice[y]->text().toInt(); z++) {
               r += t.sprintf(" %i", rnd(sides[y]->text().toInt()-1)+1);
            }
         }
      }

      result += r;   
      display->append(r);
      emit rollResult(r);
   }

   // FIXME: reinstate this when RollAction is converted
   /*
   RollAction *a = new RollAction(result);
   Game->done.log(a);
   */   

   // FIXME: eliminate in-window display?
}


void RollDialog::validateRoll() {
   // Allow rolls only when:
   // 1. [1, 1000] sets will be rolled
   // 2. [1, 1000] dice will be rolled for each checked row
   // 3. at least one row is checked
   // 3. all dice to be rolled have [1, 1000] faces

   if (!sets->text().toUInt() || sets->text().toUInt() > 1000) {
      roll->setEnabled(false);
      return;
   }

   bool checked = false;
   
   for (int x = 0; x < 3; x++) {
      if (cb[x]->isChecked()) {
         unsigned int d = dice[x]->text().toUInt();
         unsigned int s = dice[x]->text().toUInt();
         
         if (!d || !s || d > 1000 || s > 1000) {   
            roll->setEnabled(false);
            return;        
         }
         
         checked = true;
      }      
   }

   if (!checked) {
      roll->setEnabled(false);
      return;
   }
   
   roll->setEnabled(true);
}


int RollDialog::rnd(int max) {
   int r = RAND_MAX % (max+1);
   int m = (r == max) ? RAND_MAX : RAND_MAX-r-1;

   int n;
   while ((n = rand()) > m);

   return n % (max+1);
}


//
// RValidator
//
RValidator::RValidator(unsigned int b, unsigned int t, QWidget *parent,
 const char *name) : QValidator(parent, name), bottom(b), top(t) {
}


RValidator::State RValidator::validate(QString &input, int &) const {
   if (input.find(QRegExp("[^\\d]")) != -1) return Invalid;
   if (input.toUInt() < bottom || input.toUInt() > top) return Intermediate;
   return Acceptable;
}
