// $Id: surface.h,v 1.1 2004/07/15 16:43:07 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2001, 2002, 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __SURFACE_H_
#define __SURFACE_H_

#include <qdom.h>
#include <qfileinfo.h>

#ifdef Q_WS_WIN
#include <qimage.h>
#endif 

#include "view.h"

class GameObj;


class Surface : public View {
   Q_OBJECT        

   public:
#ifdef Q_WS_WIN
      Surface(GameObj *g, QCanvas *c, const QString &n, const QFileInfo &b,
       const QImage p = QImage(), unsigned int i = 0, QWidget *parent = 0,
       const char* name = 0, WFlags f = 0);
#else
      Surface(GameObj *g, QCanvas *c, const QString &n, const QFileInfo &b,
       const QPixmap p = QPixmap(), unsigned int i = 0, QWidget *parent = 0,
       const char* name = 0, WFlags f = 0);
#endif
      Surface(const Surface &);
      void setSurfaceName(const QString &);
      QString surfaceName() const;
      void setBackFile(const QFileInfo &);
      QFileInfo backFile() const;
      unsigned int id() const;   
      QDomElement dumpXML(QDomDocument &) const;  
 
   protected slots:
      virtual void showEvent(QShowEvent *);
      virtual void hideEvent(QHideEvent *); 
      virtual void mousePressEvent(QMouseEvent *);
      virtual void resizeEvent(QResizeEvent *);
     
   signals:
      void pressed(Surface *);
      void hidden(Surface *);
      void shown(Surface *);
      
   private:
      QString Name;
      QFileInfo bgfile;
      unsigned int ID;
};


#endif /* __SURFACE_H_ */
