// $Id: surfacedialogs.h,v 1.2 2004/07/23 04:28:09 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __SURFACEDIALOGS_H_
#define __SURFACEDIALOGS_H_

#include <qlistview.h>
#include <qpushbutton.h>

#include "bijection.h"
#include "editview.h"
#include "surface.h"
#include "tabpage.h"

class GameObj;


class SurfaceView : public EditView {
   Q_OBJECT

   friend class SurfaceViewCheckItem;

   public:
      SurfaceView(GameObj *g, QWidget *parent = 0, const char *name = 0);
      
   signals:
      void toggled(QCheckListItem *i);

   protected slots:
      virtual void rename();

   private:
      GameObj *game;
};


class SurfaceViewCheckItem : public QCheckListItem {
   friend class SurfaceView;

   public:
      SurfaceViewCheckItem(SurfaceView *parent, const QString &text);
      ~SurfaceViewCheckItem();

   protected:
      virtual void activate();
};


class SurfaceListDialog : public TabPage {
   Q_OBJECT   

   friend class SurfaceView;

   public:
      SurfaceListDialog(GameObj *g, QWidget *parent = 0, const char *name = 0,
       WFlags f = 0);

   public slots:
      void insert(Surface *s);
      void remove(Surface *s);
      void rename(Surface *s);

   private slots:
      void toggleVisibility(Surface *s);
      void toggleVisibility(QCheckListItem *i);
      void setActive(Surface *);
      void raiseActive(QListViewItem *i);
      void createSurface();
      void deleteSurface();
      void copySurface();
      void changedSelection(QListViewItem *i);

   private:
      SurfaceView *view;
      QPushButton *nb, *tb, *cb;
      Bijection<Surface *, QCheckListItem *> s2i;
      GameObj *game;
};


#endif /* __SURFACEDIALOGS_H_ */
