// $Id: tabdialog.h,v 1.11 2004/07/22 06:29:19 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __TABDIALOG_H_
#define __TABDIALOG_H_

#include <qdragobject.h>
#include <qtabbar.h>
#include <qtabwidget.h>

class TabDialog;
class TabPage;


class TabDrag : public QStoredDrag {
   Q_OBJECT
   
   public:
      TabDrag(TabPage *page, QWidget *dragSource = 0, const char *name = 0);
      static bool canDecode(const QMimeSource *m);
      static bool decode(const QMimeSource *m, TabPage *&page);
};


class TabHolder : public QTabWidget {
   Q_OBJECT
  
   friend class TabBar; 
   friend class TabDialog;

   public:
      TabHolder(QWidget *parent = 0, const char *name = 0);
      virtual void addPage(TabPage *page);

   public slots:
      virtual void removePage(TabPage *page, bool autoClose = true);

   protected:
      virtual void dragEnterEvent(QDragEnterEvent *e);
      virtual void dropEvent(QDropEvent *e);
      virtual void mouseMoveEvent(QMouseEvent *e);
      virtual void mousePressEvent(QMouseEvent *e);
      virtual void mouseReleaseEvent(QMouseEvent *e);

   private:
      bool pressed;
      QPoint presspos;
};


class TabBar : public QTabBar {
   Q_OBJECT

   friend class TabHolder;

   public:
      TabBar(QWidget *parent = 0, const char *name = 0);

   protected:
      virtual void dragEnterEvent(QDragEnterEvent *e);
      virtual void dropEvent(QDropEvent *e);
      virtual void mouseMoveEvent(QMouseEvent *e);
      virtual void mousePressEvent(QMouseEvent *e);
      virtual void mouseReleaseEvent(QMouseEvent *e);
};


class TabDialog : public QWidget {
   Q_OBJECT

   public:
      TabDialog(QWidget *parent = 0, const char *name = 0,
       WFlags f = WDestructiveClose);
      ~TabDialog();  
      void addPage(TabPage *page);
 
   protected:
      virtual void dragEnterEvent(QDragEnterEvent *e);
      virtual void dropEvent(QDropEvent *e);

   private:
      TabHolder *tw;
};

#endif /* __TABDIALOG_H_ */
