// $Id: tabpage.cpp,v 1.3 2004/06/29 00:07:47 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qtooltip.h>

#include "tabpage.h"

#include "icons/close.xpm"


//
// TabPage
//
TabPage::TabPage(const QIconSet &i, const QString &t, QWidget *parent,
 const char *name, WFlags f) : QWidget(parent, name, f), IconSet(i),
 ToolTip(t) {
   // page layout
   QVBoxLayout *vl = new QVBoxLayout(this, 5);
 
   // header layout
   QHBoxLayout *tl = new QHBoxLayout(vl);
  
   // label
   QLabel *label = new QLabel(t, this);
   tl->addWidget(label);
   tl->addStretch();

   // close button
   QPushButton *close = new QPushButton(this);
   close->setPixmap(QPixmap((const char **)close_xpm));
   close->setFixedSize(15, 15);
   close->setFocusPolicy(QWidget::NoFocus);
   QToolTip::add(close, "Close this Tab");
   connect(close, SIGNAL(clicked()), this, SLOT(closeRequest()));
   tl->addWidget(close);
}


QIconSet TabPage::iconSet() const { return IconSet; }


void TabPage::setIconSet(const QIconSet &i) { IconSet = i; }


QString TabPage::toolTip() const { return ToolTip; }


void TabPage::setToolTip(const QString &t) { ToolTip = t; }


void TabPage::closeRequest() {
   emit requestClose(this);
}
