// $Id: token.h,v 1.9 2004/08/05 21:38:30 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2001, 2002, 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __TOKEN_H_
#define __TOKEN_H_

#include <qcanvas.h>
#include <qpixmap.h>
#include <qwmatrix.h>

#include "group.h"

class GameObj;


class TokenPixmap : public QPixmap {
   public:
      TokenPixmap();
      TokenPixmap(unsigned int id);
      TokenPixmap(const QPixmap &pix, unsigned int id);
      unsigned int id() const;

   private:
      unsigned int pixid;
};


typedef QValueList<TokenPixmap> TokenPixmapList;


struct TokenType {
   unsigned int type;
   TokenPixmapList side;
   QString name;
};


class Token : public QCanvasPolygon, public Groupable {
   public:
      Token(GameObj *g, Group *gr, QCanvas *canvas, int x, int y, int z, int a,
       const QString &n, unsigned int t, unsigned int u, unsigned int i);
      Token(const Token &t, unsigned int i);
      ~Token();
      virtual void show();
      virtual void hide();
      virtual bool isVisible() const;
      virtual int rtti() const;
      virtual void setX(double x);
      virtual void setY(double y);
      virtual void moveBy(double dx, double dy);
      virtual void setSelected(bool selected);
      unsigned int type() const;
      unsigned int up() const;
      QPixmap face() const; 
      void flipNext();
      void flipPrev();
      int theta() const;
      void setTheta(int t);
      virtual bool permission(const Player *pl, unsigned int pm) const;
      void advanceAnts(unsigned int a);
      virtual QDomElement dumpXML(QDomDocument &doc) const;     

      static const int RTTI = 500000; 

   protected:
      QWMatrix transMatrix(double x, double y, double t) const;
      void adjustCorners();
      void drawShape(QPainter &);
      
   private:
      TokenPixmapList pl;
      TokenPixmapList::const_iterator side;
      unsigned int Type;
      int Theta;
      QWMatrix m;
      QPixmap r;
      QPointArray selection;
};


#endif /* __TOKEN_H_ */
