// $Id: tokendialogs.h,v 1.5 2004/07/31 01:13:17 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __TOKENDIALOGS_H_
#define __TOKENDIALOGS_H_

#include <qlistbox.h>
#include <qlistview.h>
#include <qmenudata.h>

#include "bijection.h"
#include "token.h"
#include "tabpage.h"

struct TokenType;


class TokenListBoxItem : public QListBoxItem {
   public:
      TokenListBoxItem(TokenType *, QListBox *listbox = 0);
      virtual int width(const QListBox *) const;
      virtual int height(const QListBox *) const;
      TokenType *definition() const;
      
   protected:
      virtual void paint(QPainter *);

   private:
      TokenType *def;
};


class TokenListBox : public QListBox {
   Q_OBJECT
        
   public:
      TokenListBox(QWidget *parent = 0, const char *name = 0, WFlags f = 0);
      void append(TokenListBoxItem *);     
 
   protected:
      virtual void viewportMousePressEvent(QMouseEvent *);
      virtual void viewportMouseMoveEvent(QMouseEvent *);
      
   private:
      TokenListBoxItem *moving;
};


class TokenListDialog : public TabPage {
   Q_OBJECT

   public:
      TokenListDialog(QWidget *parent = 0, const char *name = 0, WFlags f = 0);
      void append(TokenType *type);

   private:
      TokenListBox *listBox;
};


class TokenListView : public QListView {
   Q_OBJECT
   
   public:
      TokenListView(GameObj *, QWidget *parent = 0, const char *name = 0);
      void insert(Token *t); 
      void remove(Token *t);
      void update(unsigned int column, Token *t);
      virtual void clear();

   private:
      Bijection<Token *, QListViewItem *> t2i;
      GameObj *game;
};


class TokenListViewDialog : public TabPage {
   Q_OBJECT

   public:
      TokenListViewDialog(GameObj *g, const QIconSet &icon,
       const QString &title, QWidget *parent = 0, const char *name = 0,
       WFlags f = 0);
      void insert(Token *c);
      void remove(Token *c);
      void update(unsigned int column, Token *c);
      void clear();

   private:
      TokenListView *view;
};


#endif /* __TOKENDIALOGS_H_ */
