// $Id: tokendrag.cpp,v 1.5 2004/08/01 02:15:11 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <qdatastream.h>

#include "cil-idl.h"
#include "token.h"
#include "tokendrag.h"

extern QString TokenMIMEtype;


//
// TokenDrag
//
TokenDrag::TokenDrag(Token *c, const QPoint &p, QWidget *dragSource,
 const char *name) : QStoredDrag(TokenMIMEtype, dragSource, name) {
   QByteArray bytes;
   QDataStream str(bytes, IO_WriteOnly);

   QValueList<unsigned int> tokens;
   QValueList<QPoint> points;

   tokens << c->id();
   points << p;

   str << tokens << points;

   setEncodedData(bytes);
}


TokenDrag::TokenDrag(const QCanvasItemList &tokens,
 const QValueList<QPoint> &points, QWidget *dragSource, const char *name) :
 QStoredDrag(TokenMIMEtype, dragSource, name) {   
   QByteArray bytes;
   QDataStream str(bytes, IO_WriteOnly);

   str << cil2idl(tokens) << points;

   setEncodedData(bytes);
}


TokenDrag::TokenDrag(const QValueList<unsigned int> &tokens,
 const QValueList<QPoint> &points, QWidget *dragSource, const char *name) :
 QStoredDrag(TokenMIMEtype, dragSource, name) {
   QByteArray bytes;
   QDataStream str(bytes, IO_WriteOnly);

   // encode each listed token
   str << tokens << points;

   setEncodedData(bytes);
}


TokenDrag::TokenDrag(const QByteArray &bytes, QWidget *dragSource,
 const char *name) : QStoredDrag(TokenMIMEtype, dragSource, name) {
   setEncodedData(bytes);
}


TokenDrag::~TokenDrag() {
   // drop failed if no target
   if (!target()) emit failed();
   // must clear target since it's static
   setTarget(0);
}


bool TokenDrag::canDecode(const QMimeSource *m) {
   return m->provides(TokenMIMEtype);
}


bool TokenDrag::decode(const QMimeSource *m, QCanvasItemList &tokens,
 QValueList<QPoint> &points) {
   QDataStream str(m->encodedData(TokenMIMEtype), IO_ReadOnly);

   QValueList<unsigned int> vl;
   
   str >> vl >> points;
   tokens = idl2cil(vl);

   return true;
}


bool TokenDrag::decode(const QMimeSource *m, QValueList<unsigned int> &tokens,
 QValueList<QPoint> &points) {
   QDataStream str(m->encodedData(TokenMIMEtype), IO_ReadOnly);
   
   str >> tokens >> points;

   return true;
}
