// $Id: view.h,v 1.65 2004/09/21 15:13:09 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2002, 2003, 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __VIEW_H_
#define __VIEW_H_

#include <qaccel.h>
#include <qcanvas.h>
#include <qmap.h>
#include <qpopupmenu.h>
#include <qrect.h>
#include <qtimer.h>

#include "common.h"

class Action;
class ComboAction;
class Polyline;
class GameObj;
class Token;
class FreehandItem;
class Preference;


class View : public QCanvasView {
   Q_OBJECT

   public:
      View(GameObj *g, QCanvas *c, QWidget *parent = 0, const char *name = 0,
       WFlags f = 0); 
      //View(const View &);
      ~View();
      void normalizeTokenHeights();
      double zoom() const;
      void setName(const QString &);
      QString name() const;
      static bool usesPref(const QString &);
      QValueList<Token *> tokens() const;
      
   public slots:
      void setPlotting(FreehandItem *, QCanvasItemList);
      void setPasteable(bool);
      
   protected:
      virtual void keyPressEvent(QKeyEvent *);
      virtual void keyReleaseEvent(QKeyEvent *); 
      virtual void contentsMousePressEvent(QMouseEvent *);
      virtual void contentsMouseReleaseEvent(QMouseEvent *);
      virtual void contentsMouseMoveEvent(QMouseEvent *);
      virtual void contentsDragEnterEvent(QDragEnterEvent *);
      virtual void contentsDropEvent(QDropEvent *);
      //virtual void drawContents(QPainter *, int, int, int, int); 
      
      GameObj *game;

   private slots:
      void cut();
      void copy();
      void paste();
      void selectAll();
      void selectNone();
      void selectInvert();
      void stackOffset();
      void stackFlush();
      void stackShuffle();
      void stackInvert();
      void properties();
      void scrollUp();
      void scrollDown();
      void scrollLeft();
      void scrollRight();
      void doAutoScroll();
      void rebindKey(const QString binding);
      void setSelected(bool enable);
      
   private:
      void clearClipboard();

      QCanvasItemList collisionsRecursive(const QCanvasItem *,
       QCanvasItemList) const;
      void startAutoScroll();
      void stopAutoScroll();
      void clipMovement(QPoint &, bool *drag = 0, bool *scroll = 0);
      void endMovement();
    
      QCanvasItemList removeNonTokens(QCanvasItemList l) const;
  
      void cut(QCanvasItemList list);
      void copy(QCanvasItemList list);

      void stack(QCanvasItemList l, const QPoint &offset);

      void stackShuffle(QCanvasItemList l);
      void stackInvert(QCanvasItemList l);

      void logAction(Action *a);
 
      void toolMove(QMouseEvent *e);
      void toolClone(QMouseEvent *e);
      void toolPlot(QMouseEvent *e);
      void toolTrash(QMouseEvent *e);
      void toolFlipLeft(QMouseEvent *e);
      void toolFlipRight(QMouseEvent *e);
      void toolRotateCW(QMouseEvent *e);
      void toolRotateCCW(QMouseEvent *e);
      void toolRegroup(QMouseEvent *e);
      void toolSelect(QMouseEvent *e);
      void toolWand(QMouseEvent *e);
      void toolCrayon(QMouseEvent *e);
      void toolZoomIn(QMouseEvent *e);
      void toolZoomOut(QMouseEvent *e);
      void toolProp(QMouseEvent *e);
      
      void toolRaise(QMouseEvent *e);
      
      void (View::*toolFunc[TLastTool])(QMouseEvent *);
     
      QCanvasItem *raised, *selecting, *freehand, *plot;
      QCanvasItemList moving, plotting, collisions;
      Polyline *pline;
      QTimer scroll_timer;
      QPoint moving_start;
      bool compound;
      double fall, max_height, Zoom, scroll_move;
      QPopupMenu menu;
      QAccel accel;

      QMap<QString, int> binding2key;
      
      static const QCursor *NormalCursor, *PlusCursor, *MinusCursor,
       *UnionCursor, *IntersectCursor;

      static QMap<QString,char*> Slot;
       
      enum { mUndo, mRedo,
             mCut, mCopy, mPaste,
             mSelectInvert, mSelectAll, mSelectNone,
             mStackOffset, mStackFlush, mStackShuffle, mStackInvert,
             mProperties
      };
};

#endif /* __VIEW_H_ */
