// $Id: xmlhandler.cpp,v 1.1 2004/08/24 14:53:39 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <iostream>
using namespace std;


#include "xmlhandler.h"


XmlHandler::XmlHandler(const QString &file) : parseFile(file) {
}


QString XmlHandler::errorString() { return error; }


bool XmlHandler::fatalError(const QXmlParseException &exception) {
   cerr << QString("Error: %1, in %2:%3,%4!")
    .arg(exception.message()).arg(parseFile).arg(exception.lineNumber())
    .arg(exception.columnNumber()) << endl;
   return false;
}


bool XmlHandler::setError(ErrorType type, const QString &one,
 const QString &two, const QString &three, const QString &four) {
   // set the base error message
   switch (type) {
      case AttributeMissing:
         error = "'%1' element lacks '%2' attribute";
         break;
      case AttributeNotBool:
         error = "'%2' value for '%1' element is not a bool";
         break;
      case AttributeNotInt:
         error = "'%2' value for '%1' element is not an integer";
         break;
      case AttributeNotUInt:
         error = "'%2' value for '%1' element is not a nonnegative integer";
         break;
      case AttributeNotHex:
         error = "'%2' value for '%1' element is not hexadecimal";
         break;
      case AttributeNotUnique:
         error = "'%2' value for '%1' element is not unique";
         break;
      case AttributeNotMatched:
         error = "'%2' value for '%1' element has no match";
         break;
      case ElementBadSibling:
        error = "'%1' element may succeed the only following elements: %2";
         break;
      case ElementBadParent:
         error =
          "'%1' element may be a child of only the following elements: %2";
         break;
      case ElementNotRoot:
         error = "'%1' element must be the root element";
         break;
      case ElementUnrecognized:
         error = "'%1' element is unrecognized";
         break;
      case FileReadFailed:
         error = "'%1' could not be read";
         break;
      case ValuesNotEqual:
         error = "%1 '%2' != %3 '%4'";
         break;
      case ErrorVerbatim:
         error = "%1";
         break;
      default:
         error = "D'oh!";
         break;
   }

   // interpolate args
   error = error.
    replace("%1", one).
    replace("%2", two).
    replace("%3", three).
    replace("%4", four);

   return false;
}


QMap<QString,QString> XmlHandler::attributes(const QXmlAttributes &atts)
 const {
   QMap<QString,QString> map;
   for (int i = 0; i < atts.length(); ++i)
    map[atts.qName(i)] = atts.value(i);
   return map;
}


XmlHandler::ErrorType XmlHandler::validateBool(
 const QMap<QString,QString> &attr, const QString &key, bool &val) {
   // Does the key exist?
   if(!attr.contains(key)) return AttributeMissing;

   // Is its value an unsigned int?
   bool ok = false;
   bool tmp = (bool)(attr[key].toUInt(&ok));
   if (!ok) return AttributeNotBool;

   val = tmp;
   return Ok;
}


XmlHandler::ErrorType XmlHandler::validateInt(
 const QMap<QString,QString> &attr, const QString &key, int &val) {
   // Does the key exist?
   if(!attr.contains(key)) return AttributeMissing;

   // Is its value an unsigned int?
   bool ok = false;
   int tmp = attr[key].toInt(&ok);
   if (!ok) return AttributeNotInt;

   val = tmp;
   return Ok;
}


XmlHandler::ErrorType XmlHandler::validateUInt(
 const QMap<QString,QString> &attr, const QString &key, unsigned int &val) {
   // Does the key exist?
   if(!attr.contains(key)) return AttributeMissing;

   // Is its value an unsigned int?
   bool ok = false;
   unsigned int tmp = attr[key].toUInt(&ok);
   if (!ok) return AttributeNotUInt;

   val = tmp;
   return Ok;
}
