// $Id: xmlhandler.h,v 1.1 2004/08/24 14:53:39 uckelman Exp $
/* Board -- a graphical frontend for boardgames
 * Copyright (C) 2004 Joel Uckelman
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __XMLHANDLER_H_
#define __XMLHANDLER_H_

#include <qmap.h>
#include <qvaluestack.h>
#include <qxml.h>


class XmlHandler : public QXmlDefaultHandler {
   public:
      XmlHandler(const QString &file);
      virtual bool startElement(const QString &namespaceURI,
       const QString &localName, const QString &qName,
       const QXmlAttributes &atts) = 0;
      virtual QString errorString();
      virtual bool fatalError(const QXmlParseException &exception);

   protected:
      typedef enum {
         Ok, ErrorVerbatim,

         AttributeNotBool, AttributeNotInt, AttributeNotUInt, AttributeNotHex,
         AttributeMissing, AttributeNotUnique, AttributeNotMatched,

         ElementBadSibling, ElementBadParent, ElementUnrecognized,
         ElementNotRoot,

         FileReadFailed,

         ValuesNotEqual
      } ErrorType;

      bool setError(ErrorType type, const QString &one = QString::null,
       const QString &two = QString::null,
       const QString &three = QString::null,
       const QString &four = QString::null);

      QMap<QString,QString> attributes(const QXmlAttributes &atts) const;

      ErrorType validateBool(const QMap<QString,QString> &attr,
       const QString &key, bool &val);
      ErrorType validateInt(const QMap<QString,QString> &attr,
       const QString &key, int &val);
      ErrorType validateUInt(const QMap<QString,QString> &attr,
       const QString &key, unsigned int &val);

      QValueStack<QString> elementStack;
      QString prevSibling, error, parseFile;
};


#endif /* __XMLHANDLER_H_ */
